@@ -50,8 +50,8 @@ do_cl_join_query(long long action,
 
 /*	 A_CL_JOIN_ANNOUNCE	*/
 
-/* this is kind of a workaround for the fact that we may not be around
- * or are otherwise unable to reply when the DC sends out A_WELCOME_ALL
+/* this is kind of a workaround for the fact that we may not be around or
+ * are otherwise unable to reply when the DC sends out A_DC_JOIN_OFFER_ALL
  */
 void
 do_cl_join_announce(long long action,
@@ -65,9 +65,10 @@ do_cl_join_announce(long long action,
      * left the CCM list
      */
 
-    /* dont announce if we're in one of these states */
+    /* don't announce if we're in one of these states */
     if (cur_state != S_PENDING) {
-        crm_warn("Do not announce ourselves in state %s", fsa_state2string(cur_state));
+        crm_warn("Not announcing cluster join because in state %s",
+                 fsa_state2string(cur_state));
         return;
     }
 
@@ -83,7 +84,7 @@ do_cl_join_announce(long long action,
 
     } else {
         /* Delay announce until we have finished local startup */
-        crm_warn("Delaying announce until local startup is complete");
+        crm_warn("Delaying announce of cluster join until local startup is complete");
         return;
     }
 }
@@ -111,7 +112,7 @@ do_cl_join_offer_respond(long long action,
     }
 #endif
 
-    crm_trace("Accepting join offer from %s: join-%s",
+    crm_trace("Accepting cluster join offer from node %s "CRM_XS" join-%s",
               welcome_from, crm_element_value(input->msg, F_CRM_JOIN_ID));
 
     /* we only ever want the last one */
@@ -122,7 +123,8 @@ do_cl_join_offer_respond(long long action,
     }
 
     if (update_dc(input->msg) == FALSE) {
-        crm_warn("Discarding offer from %s (expected %s)", welcome_from, fsa_our_dc);
+        crm_warn("Discarding cluster join offer from node %s (expected %s)",
+                 welcome_from, fsa_our_dc);
         return;
     }
 
@@ -206,8 +208,8 @@ do_cl_join_finalize_respond(long long action,
     crm_element_value_int(input->msg, F_CRM_JOIN_ID, &join_id);
 
     if (was_nack) {
-        crm_err("Join (join-%d) with leader %s failed (NACK'd): Shutting down",
-                join_id, welcome_from);
+        crm_err("Shutting down because cluster join with leader %s failed "
+                CRM_XS" join-%d NACK'd", welcome_from, join_id);
         register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
         return;
     }
@@ -218,12 +220,12 @@ do_cl_join_finalize_respond(long long action,
     }
 
     if (update_dc(input->msg) == FALSE) {
-        crm_warn("Discarding %s from %s (expected %s)", op, welcome_from, fsa_our_dc);
+        crm_warn("Discarding %s from node %s (expected from %s)",
+                 op, welcome_from, fsa_our_dc);
         return;
     }
 
     /* send our status section to the DC */
-    crm_debug("Confirming join join-%d: %s", join_id, crm_element_value(input->msg, F_CRM_TASK));
     tmp1 = do_lrm_query(TRUE, fsa_our_uname);
     if (tmp1 != NULL) {
         xmlNode *reply = create_request(CRM_OP_JOIN_CONFIRM, tmp1, fsa_our_dc,
@@ -231,36 +233,26 @@ do_cl_join_finalize_respond(long long action,
 
         crm_xml_add_int(reply, F_CRM_JOIN_ID, join_id);
 
-        crm_debug("join-%d: Join complete."
-                  "  Sending local LRM status to %s", join_id, fsa_our_dc);
+        crm_debug("Confirming join-%d: sending local operation history to %s",
+                  join_id, fsa_our_dc);
 
-        if (first_join) {
+        /*
+         * If this is the node's first join since the crmd started on it, clear
+         * any previous transient node attributes, to handle the case where
+         * the node restarted so quickly that the cluster layer didn't notice.
+         *
+         * Do not remove the resources though, they'll be cleaned up in
+         * do_dc_join_ack(). Removing them here creates a race condition if the
+         * crmd is being recovered. Instead of a list of active resources from
+         * the lrmd, we may end up with a blank status section. If we are _NOT_
+         * lucky, we will probe for the "wrong" instance of anonymous clones and
+         * end up with multiple active instances on the machine.
+         */
+        if (first_join && is_not_set(fsa_input_register, R_SHUTDOWN)) {
             first_join = FALSE;
-
-            /*
-             * Clear any previous transient node attribute and lrm operations
-             *
-             * Corosync has a nasty habit of not being able to tell if a
-             *   node is returning or didn't leave in the first place.
-             * This confuses Pacemaker because it never gets a "node up"
-             *   event which is normally used to clean up the status section.
-             *
-             * Do not remove the resources though, they'll be cleaned up in
-             *   do_dc_join_ack().  Removing them here creates a race
-             *   condition if the crmd is being recovered.
-             * Instead of a list of active resources from the lrmd
-             *   we may end up with a blank status section.
-             * If we are _NOT_ lucky, we will probe for the "wrong" instance
-             *   of anonymous clones and end up with multiple active
-             *   instances on the machine.
-             */
             erase_status_tag(fsa_our_uname, XML_TAG_TRANSIENT_NODEATTRS, 0);
-
-            /* Just in case attrd was still around too */
-            if (is_not_set(fsa_input_register, R_SHUTDOWN)) {
-                update_attrd(fsa_our_uname, "terminate", NULL, NULL, FALSE);
-                update_attrd(fsa_our_uname, XML_CIB_ATTR_SHUTDOWN, "0", NULL, FALSE);
-            }
+            update_attrd(fsa_our_uname, "terminate", NULL, NULL, FALSE);
+            update_attrd(fsa_our_uname, XML_CIB_ATTR_SHUTDOWN, "0", NULL, FALSE);
         }
 
         send_cluster_message(crm_get_peer(0, fsa_our_dc), crm_msg_crmd, reply, TRUE);
@@ -274,7 +266,8 @@ do_cl_join_finalize_respond(long long action,
         free_xml(tmp1);
 
     } else {
-        crm_err("Could not send our LRM state to the DC");
+        crm_err("Could not confirm join-%d with %s: Local operation history failed",
+                join_id, fsa_our_dc);
         register_fsa_error(C_FSA_INTERNAL, I_FAIL, NULL);
     }
 }
