@@ -2,7 +2,7 @@
  * Copyright (C) 2015 Andrew Beekhof <andrew@beekhof.net>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
@@ -22,12 +22,17 @@
 #  include <crm/cluster.h>
 #  include <crm/stonith-ng.h>
 
-/* Timeout to use before killing a notification script (in milliseconds) */
-#  define CRMD_NOTIFY_TIMEOUT_MS (300000)
+/* Default-Timeout to use before killing a notification script (in milliseconds) */
+#  define CRMD_NOTIFY_DEFAULT_TIMEOUT_MS (30000)
+
+/* Default-Format-String used to pass timestamps to the notification scripts */
+#  define CRMD_NOTIFY_DEFAULT_TSTAMP_FORMAT "%H:%M:%S.%06N"
 
 void crmd_enable_notifications(const char *script, const char *target);
 void crmd_notify_node_event(crm_node_t *node);
-void crmd_notify_fencing_op(stonith_event_t * e);
-void crmd_notify_resource_op(const char *node, lrmd_event_data_t * op);
+void crmd_notify_fencing_op(stonith_event_t *e);
+void crmd_notify_resource_op(const char *node, lrmd_event_data_t *op);
+void crmd_drain_alerts(GMainContext *ctx);
+void parse_notifications(xmlNode *notifications);
 
 #endif
