@@ -56,12 +56,12 @@ class BasePatterns:
             "Pat:They_dead"     : "node %s.*: is dead",
             "Pat:TransitionComplete" : "Transition status: Complete: complete",
 
-            "Pat:Fencing_start" : "Initiating remote operation .* for %s",
+            "Pat:Fencing_start" : "(Initiating remote operation|Requesting peer fencing ).* (for|of) %s",
             "Pat:Fencing_ok"    : r"stonith.*:\s*Operation .* of %s by .* for .*@.*: OK",
             "Pat:Fencing_recover"    : r"pengine.*: Recover %s",
 
-            "Pat:RscOpOK"       : r"crmd.*:\s*Operation %s_%s.*:\s*ok \(.*confirmed=\S+\)",
-            "Pat:RscRemoteOpOK" : r"crmd.*:\s*Operation %s_%s.*:\s*ok \(node=%s,.*,\s*confirmed=true\)",
+            "Pat:RscOpOK"       : r"crmd.*:\s+Result of %s operation for %s.*: (0 \()?ok",
+            "Pat:RscRemoteOpOK" : r"crmd.*:\s+Result of %s operation for %s on %s: (0 \()?ok",
             "Pat:NodeFenced"    : r"crmd.*:\s*Peer\s+%s\s+was\s+terminated\s+\(.*\)\s+by\s+.*\s+for\s+.*:\s+OK",
             "Pat:FenceOpOK"     : "Operation .* for host '%s' with device .* returned: 0",
         }
@@ -168,7 +168,7 @@ class crm_cs_v0(BasePatterns):
 # The next pattern would be preferred, but it doesn't always come out
 #            "Pat:We_stopped"   : "%s.*Corosync Cluster Engine exiting with status",
             "Pat:We_stopped"   : "%s\W.*Service engine unloaded: corosync cluster quorum service",
-            "Pat:They_stopped" : "%s\W.*crmd.*Node %s\[.*state is now lost",
+            "Pat:They_stopped" : "%s\W.*crmd.*Node %s(\[|\s).*state is now lost",
             "Pat:They_dead"    : "corosync:.*Node %s is now: lost",
 
             "Pat:ChildExit"    : "Child process .* exited",
@@ -268,6 +268,7 @@ class crm_cs_v0(BasePatterns):
                     "verify_stopped:.*Resource .* was active at shutdown.  You may ignore this error if it is unmanaged.",
                     "error: attrd_connection_destroy:.*Lost connection to attrd",
                     r".*:\s*Executing .* fencing operation \(.*\) on ",
+                    r".*:\s*Requesting fencing \([^)]+\) of node ",
                     r"(Blackbox dump requested|Problem detected)",
 #                    "error: native_create_actions: Resource .*stonith::.* is active on 2 nodes attempting recovery",
 #                    "error: process_pe_message: Transition .* ERRORs found during PE processing",
@@ -289,8 +290,8 @@ class crm_cs_v0(BasePatterns):
             r"lrmd.*error:.*Connection to stonith-ng.* closed",
             r"lrmd.*error:.*LRMD lost STONITH connection",
             r"crmd.*State transition .* S_RECOVERY",
-            r"crmd.*error:.*FSA: Input I_ERROR",
-            r"crmd.*error:.*FSA: Input I_TERMINATE",
+            r"crmd.*error:.*Input I_ERROR .*received in state",
+            r"crmd.*error:.*Input I_TERMINATE .*received in state",
             r"crmd.*error:.*Connection to cman failed",
             r"crmd.*error:.*Could not recover from internal error",
             r"error:.*Connection to cib_shm failed",
@@ -304,11 +305,11 @@ class crm_cs_v0(BasePatterns):
         self.components["corosync"] = [
             r"pacemakerd.*error:.*Connection destroyed",
             r"attrd.*:\s*crit:.*Lost connection to Corosync service",
-            r"stonith.*:\s*Corosync connection terminated",
+            r"stonith.*:\s*(Corosync connection terminated|Shutting down)",
             r"cib.*:\s*Corosync connection lost!\s+Exiting.",
-            r"crmd.*:\s*connection terminated",
+            r"crmd.*:\s*(connection terminated|Disconnected from Corosync)",
             r"pengine.*Scheduling Node .* for STONITH",
-            r"crmd.*:\s*Peer %s was terminated \(.*\) by .* for .*:\s*OK",
+            r"crmd.*:\s*Peer .* was terminated \(.*\) by .* for .*:\s*OK",
         ]
 
         self.components["cib-ignore"] = [
@@ -328,7 +329,7 @@ class crm_cs_v0(BasePatterns):
                     "(Child process|The) crmd .* exited: Generic Pacemaker error",
                     "(Child process|The) attrd .* exited: (Connection reset by peer|Transport endpoint is not connected)",
                     "Lost connection to CIB service",
-                    "crmd.*Input I_TERMINATE from do_recover",
+                    r"crmd.*: Input I_TERMINATE .*from do_recover",
                     "crmd.*I_ERROR.*crmd_cib_connection_destroy",
                     "crmd.*Could not recover from internal error",
                     ]
@@ -341,7 +342,7 @@ class crm_cs_v0(BasePatterns):
                     "Connection to lrmd.* closed",
                     "crmd.*I_ERROR.*lrm_connection_destroy",
                     "(Child process|The) crmd .* exited: Generic Pacemaker error",
-                    "crmd.*Input I_TERMINATE from do_recover",
+                    r"crmd.*: Input I_TERMINATE .*from do_recover",
                     "crmd.*Could not recover from internal error",
                     ]
         self.components["lrmd-ignore"] = []
@@ -366,7 +367,7 @@ class crm_cs_v0(BasePatterns):
                     "Connection to pengine.* closed",
                     "Connection to the Policy Engine failed",
                     "crmd.*I_ERROR.*save_cib_contents",
-                    "crmd.*Input I_TERMINATE from do_recover",
+                    r"crmd.*: Input I_TERMINATE .*from do_recover",
                     "crmd.*Could not recover from internal error",
                     ]
         self.components["pengine-ignore"] = []
@@ -386,7 +387,7 @@ class crm_cs_v0(BasePatterns):
             r"crit:.*Fencing daemon connection failed",
             r"error:.*Sign-in failed: triggered a retry",
             "STONITH connection failed, finalizing .* pending operations.",
-            r"crmd.*:\s*Operation Fencing.* Error",
+            r"crmd.*:\s+Result of .* operation for Fencing.*Error",
         ]
         self.components["stonith-ignore"].extend(self.components["common-ignore"])
 
@@ -412,8 +413,8 @@ class crm_mcp(crm_cs_v0):
             # Close enough... "Corosync Cluster Engine exiting normally" isn't printed
             #   reliably and there's little interest in doing anything about it
             "Pat:We_stopped"   : "%s\W.*Unloading all Corosync service engines",
-            "Pat:They_stopped" : "%s\W.*crmd.*Node %s\[.*state is now lost",
-            "Pat:They_dead"    : "crmd.*Node %s\[.*state is now lost",
+            "Pat:They_stopped" : "%s\W.*crmd.*Node %s(\[|\s).*state is now lost",
+            "Pat:They_dead"    : "crmd.*Node %s(\[|\s).*state is now lost",
 
             "Pat:ChildExit"    : "The .* process exited",
             "Pat:ChildKilled"  : "%s\W.*pacemakerd.*The %s process .* terminated with signal 9",
@@ -462,8 +463,8 @@ class crm_cman(crm_cs_v0):
 
         self.search.update({
             "Pat:We_stopped"   : "%s.*Unloading all Corosync service engines",
-            "Pat:They_stopped" : "%s\W.*crmd.*Node %s\[.*state is now lost",
-            "Pat:They_dead"    : "crmd.*Node %s\[.*state is now lost",
+            "Pat:They_stopped" : "%s\W.*crmd.*Node %s(\[|\s).*state is now lost",
+            "Pat:They_dead"    : "crmd.*Node %s(\[|\s).*state is now lost",
 
             "Pat:ChildKilled"  : "%s\W.*pacemakerd.*The %s process .* terminated with signal 9",
             "Pat:ChildRespawn" : "%s\W.*pacemakerd.*Respawning failed child process: %s",
