@@ -2,7 +2,7 @@
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
@@ -219,11 +219,54 @@ char *calculate_operation_digest(xmlNode * local_cib, const char *version);
 char *calculate_xml_versioned_digest(xmlNode * input, gboolean sort, gboolean do_filter,
                                      const char *version);
 
+/* schema-related functions (from schemas.c) */
 gboolean validate_xml(xmlNode * xml_blob, const char *validation, gboolean to_logs);
 gboolean validate_xml_verbose(xmlNode * xml_blob);
-int update_validation(xmlNode ** xml_blob, int *best, int max, gboolean transform, gboolean to_logs);
+
+/*!
+ * \brief Try update CIB XML to the highest pacemaker's standard if feasible
+ *
+ * "Update" means either actively employ XSLT-based transformation(s)
+ * (if intermediate product to transform valid per its declared schema version,
+ * transformation available, proceeded successfully with a result valid per
+ * expectated newer schema version), or just try to bump the marked validating
+ * schema until all gradually rising schema versions attested or the first
+ * such attempt subsequently fails to validate.   Which of the two styles will
+ * be used depends on \p transform parameter (positive/negative, respectively).
+ *
+ * \param[in/out] xml_blob   XML tree representing CIB, may be swapped with
+ *                           an "updated" one
+ * \param[out]    best       The highest configuration version (per its index
+ *                           in the global schemas table) it was possible to
+ *                           reach during the update steps while ensuring
+ *                           the validity of the result; if no validation
+ *                           success was observed against possibly multiple
+ *                           schemas, the value is less or equal the result
+ *                           of <tt>get_schema_version</tt> applied on the
+ *                           input \p xml_blob value (unless that function
+ *                           maps it to -1, then 0 would be used instead)
+ * \param[in]     max        When \p transform is positive, this allows to
+ *                           set upper boundary schema (per its index in the
+ *                           global schemas table) beyond which its forbidden
+ *                           to update by the means of XSLT transformation
+ * \param[in]     transform  Whether to employ XSLT-based transformation so
+ *                           as allow overcoming possible incompatibilities
+ *                           between major schema versions (see above)
+ * \param[in]     to_logs    If true, output notable progress info to
+ *                           internal log streams; if false, to stderr
+ *
+ * \return <tt>pcmk_ok</tt> if no non-recoverable error encountered (up to
+ *         caller to evaluate if the update satisfies the requirements
+ *         per returned \p best value), negative value carrying the reason
+ *         otherwise
+ */
+int update_validation(xmlNode **xml_blob, int *best, int max,
+                      gboolean transform, gboolean to_logs);
+
 int get_schema_version(const char *name);
 const char *get_schema_name(int version);
+const char *xml_latest_schema(void);
+gboolean cli_config_update(xmlNode ** xml, int *best_version, gboolean to_logs);
 
 void crm_xml_init(void);
 void crm_xml_cleanup(void);
@@ -274,7 +317,6 @@ xmlNode *sorted_xml(xmlNode * input, xmlNode * parent, gboolean recursive);
 xmlXPathObjectPtr xpath_search(xmlNode * xml_top, const char *path);
 void crm_foreach_xpath_result(xmlNode *xml, const char *xpath,
                               void (*helper)(xmlNode*, void*), void *user_data);
-gboolean cli_config_update(xmlNode ** xml, int *best_version, gboolean to_logs);
 xmlNode *expand_idref(xmlNode * input, xmlNode * top);
 
 void freeXpathObject(xmlXPathObjectPtr xpathObj);
@@ -289,8 +331,6 @@ static inline int numXpathResults(xmlXPathObjectPtr xpathObj)
     return xpathObj->nodesetval->nodeNr;
 }
 
-const char *xml_latest_schema(void);
-
 bool xml_acl_enabled(xmlNode *xml);
 void xml_acl_disable(xmlNode *xml);
 bool xml_acl_denied(xmlNode *xml); /* Part or all of a change was rejected */
