@@ -2,7 +2,7 @@
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
@@ -29,13 +29,11 @@
 #  include <glib.h>
 #  include <stdbool.h>
 
-#  undef MIN
-#  undef MAX
 #  include <string.h>
 
 #  include <libxml/tree.h>
 
-#  define CRM_FEATURE_SET		"3.0.10"
+#  define CRM_FEATURE_SET		"3.0.11"
 
 #  define EOS		'\0'
 #  define DIMOF(a)	((int) (sizeof(a)/sizeof(a[0])) )
@@ -71,6 +69,14 @@ extern char *crm_system_name;
 #  define DOT_ALL_FSA_INPUTS	1
 /* #define FSA_TRACE		1 */
 
+/* This header defines INFINITY, but it might be defined elsewhere as well
+ * (e.g. math.h), so undefine it first. This, of course, complicates any attempt
+ * to use the other definition in any code that includes this header.
+ *
+ * @TODO: Rename our constant (which will break API backward compatibility).
+ */
+#  undef INFINITY
+
 #  define INFINITY_S        "INFINITY"
 #  define MINUS_INFINITY_S "-INFINITY"
 
@@ -132,6 +138,7 @@ extern char *crm_system_name;
 #  define CRM_OP_NODES_PROBED	"probe_nodes_complete"
 #  define CRM_OP_REPROBE		"probe_again"
 #  define CRM_OP_CLEAR_FAILCOUNT  "clear_failcount"
+#  define CRM_OP_REMOTE_STATE     "remote_state"
 #  define CRM_OP_RELAXED_SET  "one-or-more"
 #  define CRM_OP_RELAXED_CLONE  "clone-one-or-more"
 #  define CRM_OP_RM_NODE_CACHE "rm_node_cache"
@@ -144,6 +151,7 @@ extern char *crm_system_name;
 #  define CRMD_ACTION_DELETE		"delete"
 #  define CRMD_ACTION_CANCEL		"cancel"
 
+#  define CRMD_ACTION_RELOAD		"reload"
 #  define CRMD_ACTION_MIGRATE		"migrate_to"
 #  define CRMD_ACTION_MIGRATED		"migrate_from"
 
@@ -198,4 +206,11 @@ typedef GList *GListPtr;
 guint crm_strcase_hash(gconstpointer v);
 guint g_str_hash_traditional(gconstpointer v);
 
+static inline const char *crm_action_str(const char *task, int interval) {
+    if(safe_str_eq(task, RSC_STATUS) && !interval) {
+        return "probe";
+    }
+    return task;
+}
+
 #endif
