@@ -93,7 +93,10 @@ find_nvpair_attr_delegate(cib_t * the_cib, const char *attr, const char *section
     }
 
     xpath_string = calloc(1, xpath_max);
-    CRM_CHECK(xpath_string != NULL, return -ENOMEM);
+    if (xpath_string == NULL) {
+        crm_perror(LOG_CRIT, "Could not create xpath");
+        return -ENOMEM;
+    }
 
     attr_snprintf(xpath_string, offset, xpath_max, "%.128s", get_object_path(section));
 
@@ -484,6 +487,12 @@ get_cluster_node_uuid(cib_t * the_cib, const char *uname, char **uuid)
 
     for (a_child = __xml_first_child(xml_obj); a_child != NULL; a_child = __xml_next(a_child)) {
         if (crm_str_eq((const char *)a_child->name, XML_CIB_TAG_NODE, TRUE)) {
+            const char *node_type = crm_element_value(a_child, XML_ATTR_TYPE);
+            /* Only if it's a cluster node */
+            if (safe_str_eq(node_type, "remote")) {
+                continue;
+            }
+
             child_name = crm_element_value(a_child, XML_ATTR_UNAME);
             if (safe_str_eq(uname, child_name)) {
                 child_name = ID(a_child);
@@ -508,6 +517,10 @@ query_node_uuid(cib_t * the_cib, const char *uname, char **uuid, int *is_remote_
     CRM_ASSERT(uname != NULL);
     CRM_ASSERT(uuid != NULL);
 
+    if (is_remote_node) {
+        *is_remote_node = FALSE;
+    }
+
     rc = get_cluster_node_uuid(the_cib, uname, uuid);
     if (rc != pcmk_ok) {
         crm_debug("%s is not a cluster node, checking to see if remote-node", uname);
@@ -521,7 +534,7 @@ query_node_uuid(cib_t * the_cib, const char *uname, char **uuid, int *is_remote_
     }
 
     if (rc != pcmk_ok) {
-        crm_debug("Could not map name=%s to a UUID: %s\n", uname, pcmk_strerror(rc));
+        crm_debug("Could not map name=%s to a UUID: %s", uname, pcmk_strerror(rc));
     } else {
         crm_info("Mapped %s to %s", uname, *uuid);
     }
