@@ -28,6 +28,7 @@
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
+#include <netinet/tcp.h>
 #include <netdb.h>
 
 #include <stdlib.h>
@@ -234,7 +235,7 @@ crm_send_tls(gnutls_session_t * session, const char *buf, size_t len)
     }
 
     total_send = len;
-    crm_trace("Message size: %d", len);
+    crm_trace("Message size: %llu", (unsigned long long) len);
 
     while (TRUE) {
         rc = gnutls_record_send(*session, unsent, len);
@@ -247,7 +248,7 @@ crm_send_tls(gnutls_session_t * session, const char *buf, size_t len)
             break;
 
         } else if (rc < len) {
-            crm_debug("Sent %d of %d bytes", rc, len);
+            crm_debug("Sent %d of %llu bytes", rc, (unsigned long long) len);
             len -= rc;
             unsent += rc;
         } else {
@@ -273,7 +274,8 @@ crm_send_plaintext(int sock, const char *buf, size_t len)
     }
     total_send = len;
 
-    crm_trace("Message on socket %d: size=%d", sock, len);
+    crm_trace("Message on socket %d: size=%llu",
+              sock, (unsigned long long) len);
   retry:
     rc = write(sock, unsent, len);
     if (rc < 0) {
@@ -288,7 +290,8 @@ crm_send_plaintext(int sock, const char *buf, size_t len)
         }
 
     } else if (rc < len) {
-        crm_trace("Only sent %d of %d remaining bytes", rc, len);
+        crm_trace("Only sent %d of %llu remaining bytes",
+                  rc, (unsigned long long) len);
         len -= rc;
         unsent += rc;
         goto retry;
@@ -355,7 +358,7 @@ crm_remote_send(crm_remote_t * remote, xmlNode * msg)
     header->payload_uncompressed = iov[1].iov_len;
     header->size_total = iov[0].iov_len + iov[1].iov_len;
 
-    crm_trace("Sending len[0]=%d, start=%x\n",
+    crm_trace("Sending len[0]=%d, start=%x",
               (int)iov[0].iov_len, *(int*)(void*)xml_text);
     rc = crm_remote_sendv(remote, iov, 2);
     if (rc < 0) {
@@ -383,9 +386,6 @@ crm_remote_parse_buffer(crm_remote_t * remote)
         return NULL;
     }
 
-    /* take ownership of the buffer */
-    remote->buffer_offset = 0;
-
     /* Support compression on the receiving end now, in case we ever want to add it later */
     if (header->payload_compressed) {
         int rc = 0;
@@ -421,6 +421,9 @@ crm_remote_parse_buffer(crm_remote_t * remote)
         header = crm_remote_header(remote);
     }
 
+    /* take ownership of the buffer */
+    remote->buffer_offset = 0;
+
     CRM_LOG_ASSERT(remote->buffer[sizeof(struct crm_remote_header_v0) + header->payload_uncompressed - 1] == 0);
 
     xml = string2xml(remote->buffer + header->payload_offset);
@@ -437,19 +440,21 @@ crm_remote_parse_buffer(crm_remote_t * remote)
 
 /*!
  * \internal
- * \brief Determine if a remote session has data to read
+ * \brief Wait for a remote session to have data to read
  *
- * \retval 0, timeout occurred.
- * \retval positive, data is ready to be read
- * \retval negative, session has ended
+ * \param[in] remote         Connection to check
+ * \param[in] total_timeout  Maximum time (in ms) to wait
+ *
+ * \return Positive value if ready to be read, 0 on timeout, -errno on error
  */
 int
-crm_remote_ready(crm_remote_t * remote, int timeout /* ms */ )
+crm_remote_ready(crm_remote_t *remote, int total_timeout)
 {
     struct pollfd fds = { 0, };
     int sock = 0;
     int rc = 0;
     time_t start;
+    int timeout = total_timeout;
 
 #ifdef HAVE_GNUTLS_GNUTLS_H
     if (remote->tls_session) {
@@ -480,7 +485,7 @@ crm_remote_ready(crm_remote_t * remote, int timeout /* ms */ )
          * specific timeout we are trying to honor, attempt
          * to adjust the timeout to the closest second. */
         if (errno == EINTR && (timeout > 0)) {
-            timeout = timeout - ((time(NULL) - start) * 1000);
+            timeout = total_timeout - ((time(NULL) - start) * 1000);
             if (timeout < 1000) {
                 timeout = 1000;
             }
@@ -489,7 +494,7 @@ crm_remote_ready(crm_remote_t * remote, int timeout /* ms */ )
         rc = poll(&fds, 1, timeout);
     } while (rc < 0 && errno == EINTR);
 
-    return rc;
+    return (rc < 0)? -errno : rc;
 }
 
 
@@ -518,7 +523,8 @@ crm_remote_recv_once(crm_remote_t * remote)
     /* automatically grow the buffer when needed */
     if(remote->buffer_size < read_len) {
            remote->buffer_size = 2 * read_len;
-        crm_trace("Expanding buffer to %u bytes", remote->buffer_size);
+        crm_trace("Expanding buffer to %llu bytes",
+                  (unsigned long long) remote->buffer_size);
 
         remote->buffer = realloc_safe(remote->buffer, remote->buffer_size + 1);
         CRM_ASSERT(remote->buffer != NULL);
@@ -559,28 +565,33 @@ crm_remote_recv_once(crm_remote_t * remote)
         remote->buffer_offset += rc;
         /* always null terminate buffer, the +1 to alloc always allows for this. */
         remote->buffer[remote->buffer_offset] = '\0';
-        crm_trace("Received %u more bytes, %u total", rc, remote->buffer_offset);
+        crm_trace("Received %u more bytes, %llu total",
+                  rc, (unsigned long long) remote->buffer_offset);
 
     } else if (rc == -EINTR || rc == -EAGAIN) {
         crm_trace("non-blocking, exiting read: %s (%d)", pcmk_strerror(rc), rc);
 
     } else if (rc == 0) {
-        crm_debug("EOF encoutered after %u bytes", remote->buffer_offset);
+        crm_debug("EOF encoutered after %llu bytes",
+                  (unsigned long long) remote->buffer_offset);
         return -ENOTCONN;
 
     } else {
-        crm_debug("Error receiving message after %u bytes: %s (%d)",
-                  remote->buffer_offset, pcmk_strerror(rc), rc);
+        crm_debug("Error receiving message after %llu bytes: %s (%d)",
+                  (unsigned long long) remote->buffer_offset,
+                  pcmk_strerror(rc), rc);
         return -ENOTCONN;
     }
 
     header = crm_remote_header(remote);
     if(header) {
         if(remote->buffer_offset < header->size_total) {
-            crm_trace("Read less than the advertised length: %u < %u bytes",
-                      remote->buffer_offset, header->size_total);
+            crm_trace("Read less than the advertised length: %llu < %u bytes",
+                      (unsigned long long) remote->buffer_offset,
+                      header->size_total);
         } else {
-            crm_trace("Read full message of %u bytes", remote->buffer_offset);
+            crm_trace("Read full message of %llu bytes",
+                      (unsigned long long) remote->buffer_offset);
             return remote->buffer_offset;
         }
     }
@@ -621,12 +632,8 @@ crm_remote_recv(crm_remote_t * remote, int total_timeout /*ms */ , int *disconne
             crm_err("poll timed out (%d ms) while waiting to receive msg", remaining_timeout);
             return FALSE;
 
-        } else if (rc == -EAGAIN) {
-            crm_trace("waiting for remote connection data (up to %dms)",
-                      remaining_timeout);
-
         } else if(rc < 0) {
-            crm_debug("poll() failed: %s (%d)", pcmk_strerror(rc), rc);
+            crm_debug("could not poll: %s (%d)", pcmk_strerror(rc), rc);
 
         } else {
             rc = crm_remote_recv_once(remote);
@@ -642,7 +649,7 @@ crm_remote_recv(crm_remote_t * remote, int total_timeout /*ms */ , int *disconne
             return FALSE;
         }
 
-        remaining_timeout = remaining_timeout - ((time(NULL) - start) * 1000);
+        remaining_timeout = total_timeout - ((time(NULL) - start) * 1000);
     }
 
     return FALSE;
@@ -690,7 +697,7 @@ check_connect_finished(gpointer userdata)
                 rc = -ETIMEDOUT;
             }
         }
-        crm_trace("fd %d: select failed %d connect dispatch ", rc);
+        crm_trace("fd %d: select failed %d connect dispatch ", sock, rc);
         goto dispatch_done;
     } else if (rc == 0) {
         if ((time(NULL) - cb_data->start) < (cb_data->timeout / 1000)) {
@@ -920,3 +927,83 @@ crm_remote_tcp_connect(const char *host, int port)
 {
     return crm_remote_tcp_connect_async(host, port, -1, NULL, NULL, NULL);
 }
+
+
+/* Convert a struct sockaddr address to a string, IPv4 and IPv6: */
+
+static char *
+get_ip_str(const struct sockaddr_storage * sa, char * s, size_t maxlen)
+{
+    switch(((struct sockaddr *)sa)->sa_family) {
+        case AF_INET:
+            inet_ntop(AF_INET, &(((struct sockaddr_in *)sa)->sin_addr),
+                      s, maxlen);
+            break;
+
+        case AF_INET6:
+            inet_ntop(AF_INET6, &(((struct sockaddr_in6 *)sa)->sin6_addr),
+                      s, maxlen);
+            break;
+
+        default:
+            strncpy(s, "Unknown AF", maxlen);
+            return NULL;
+    }
+
+    return s;
+}
+
+int
+crm_remote_accept(int ssock)
+{
+    int csock = 0;
+    int rc = 0;
+    int flag = 0;
+    unsigned laddr = 0;
+    struct sockaddr_storage addr;
+    char addr_str[INET6_ADDRSTRLEN];
+#ifdef TCP_USER_TIMEOUT
+    int optval;
+    long sbd_timeout = crm_get_sbd_timeout();
+#endif
+
+    /* accept the connection */
+    laddr = sizeof(addr);
+    memset(&addr, 0, sizeof(addr));
+    csock = accept(ssock, (struct sockaddr *)&addr, &laddr);
+    get_ip_str(&addr, addr_str, INET6_ADDRSTRLEN);
+    crm_info("New remote connection from %s", addr_str);
+
+    if (csock == -1) {
+        crm_err("accept socket failed");
+        return -1;
+    }
+
+    if ((flag = fcntl(csock, F_GETFL)) >= 0) {
+        if ((rc = fcntl(csock, F_SETFL, flag | O_NONBLOCK)) < 0) {
+            crm_err("fcntl() write failed");
+            close(csock);
+            return rc;
+        }
+    } else {
+        crm_err("fcntl() read failed");
+        close(csock);
+        return flag;
+    }
+
+#ifdef TCP_USER_TIMEOUT
+    if (sbd_timeout > 0) {
+        optval = sbd_timeout / 2; /* time to fail and retry before watchdog */
+        rc = setsockopt(csock, SOL_TCP, TCP_USER_TIMEOUT,
+                        &optval, sizeof(optval));
+        if (rc < 0) {
+            crm_err("setting TCP_USER_TIMEOUT (%d) on client socket failed",
+                    optval);
+            close(csock);
+            return rc;
+        }
+    }
+#endif
+
+    return csock;
+}
