@@ -300,7 +300,7 @@ stonith_api_remove_level(stonith_t * st, int options, const char *node, int leve
                                          NULL, NULL, NULL, level);
 }
 
-/*
+/*!
  * \internal
  * \brief Create XML for stonithd topology level registration request
  *
@@ -353,7 +353,11 @@ create_level_registration_xml(const char *node, const char *pattern,
 
         crm_trace("Adding %s (%dc) at offset %d", device_list->value, adding, len);
         list = realloc_safe(list, len + adding + 1);       /* +1 EOS */
-        CRM_CHECK(list != NULL, free_xml(data); return NULL);
+        if (list == NULL) {
+            crm_perror(LOG_CRIT, "Could not create device list");
+            free_xml(data);
+            return NULL;
+        }
         sprintf(list + len, "%s%s", len?",":"", device_list->value);
         len += adding;
     }
@@ -554,7 +558,7 @@ make_args(const char *agent, const char *action, const char *victim, uint32_t vi
         if (victim_nodeid) {
             char nodeid_str[33] = { 0, };
             if (snprintf(nodeid_str, 33, "%u", (unsigned int)victim_nodeid)) {
-                crm_info("For stonith action (%s) for victim %s, adding nodeid (%d) to parameters",
+                crm_info("For stonith action (%s) for victim %s, adding nodeid (%s) to parameters",
                          action, victim, nodeid_str);
                 append_const_arg("nodeid", nodeid_str, &arg_list);
             }
@@ -737,7 +741,7 @@ read_output(int fd)
         more = read(fd, buffer, READ_MAX - 1);
 
         if (more > 0) {
-            buffer[more] = 0; /* Make sure its nul-terminated for logging
+            buffer[more] = 0; /* Make sure it's nul-terminated for logging
                               * 'more' is always less than our buffer size
                               */
             output = realloc_safe(output, len + more + 1);
@@ -839,7 +843,7 @@ stonith_action_async_done(mainloop_child_t * p, pid_t pid, int core, int signo,
 static int
 internal_stonith_action_execute(stonith_action_t * action)
 {
-    int pid, status, len, rc = -EPROTO;
+    int pid, status = 0, len, rc = -EPROTO;
     int ret;
     int total = 0;
     int p_read_fd, p_write_fd;  /* parent read/write file descriptors */
@@ -1035,8 +1039,8 @@ internal_stonith_action_execute(stonith_action_t * action)
                     WTERMSIG(status));
 
         } else {
-            crm_err("call %s for %s exited abnormally. stopped=%d, continued=%d",
-                    action->action, action->agent, WIFSTOPPED(status), WIFCONTINUED(status));
+            crm_err("call %s for %s returned unexpected status %#x",
+                    action->action, action->agent, status);
         }
     }
 
@@ -2308,7 +2312,7 @@ stonith_send_command(stonith_t * stonith, const char *op, xmlNode * data, xmlNod
     crm_element_value_int(op_reply, F_STONITH_CALLID, &reply_id);
 
     if (reply_id == stonith->call_id) {
-        crm_trace("Syncronous reply %d received", reply_id);
+        crm_trace("Synchronous reply %d received", reply_id);
 
         if (crm_element_value_int(op_reply, F_STONITH_RC, &rc) != 0) {
             rc = -ENOMSG;
