@@ -91,7 +91,7 @@ cluster_status(pe_working_set_t * data_set)
    if (is_not_set(data_set->flags, pe_flag_quick_location)
        && is_not_set(data_set->flags, pe_flag_have_quorum)
        && data_set->no_quorum_policy != no_quorum_ignore) {
-        crm_notice("We do not have quorum - fencing and resource management disabled");
+        crm_warn("Fencing and resource management disabled due to lack of quorum");
     }
 
     unpack_nodes(cib_nodes, data_set);
@@ -274,7 +274,7 @@ pe_find_node_any(GListPtr nodes, const char *id, const char *uname)
     if (match) {
         return match;
     }
-    crm_trace("Looking up %s via it's uname instead", uname);
+    crm_trace("Looking up %s via its uname instead", uname);
     return pe_find_node(nodes, uname);
 }
 
