@@ -35,7 +35,6 @@ void unpack_operation(action_t * action, xmlNode * xml_obj, resource_t * contain
                       pe_working_set_t * data_set);
 static xmlNode *find_rsc_op_entry_helper(resource_t * rsc, const char *key,
                                          gboolean include_disabled);
-static gboolean is_rsc_baremetal_remote_node(resource_t *rsc, pe_working_set_t * data_set);
 
 bool pe_can_fence(pe_working_set_t * data_set, node_t *node)
 {
@@ -64,7 +63,7 @@ bool pe_can_fence(pe_working_set_t * data_set, node_t *node)
 }
 
 node_t *
-node_copy(node_t * this_node)
+node_copy(const node_t *this_node)
 {
     node_t *new_node = NULL;
 
@@ -398,7 +397,7 @@ custom_action(resource_t * rsc, char *key, const char *task,
     if (action == NULL) {
         if (save_action) {
             pe_rsc_trace(rsc, "Creating%s action %d: %s for %s on %s %d",
-                         optional ? "" : " manditory", data_set->action_id, key,
+                         optional ? "" : " mandatory", data_set->action_id, key,
                          rsc ? rsc->id : "<NULL>", on_node ? on_node->details->uname : "<NULL>", optional);
         }
 
@@ -503,7 +502,7 @@ custom_action(resource_t * rsc, char *key, const char *task,
                        action->uuid, action->node->details->uname);
             if (is_set(action->rsc->flags, pe_rsc_managed)
                 && save_action && a_task == stop_rsc) {
-                pe_fence_node(data_set, action->node, "Node is unclean");
+                pe_fence_node(data_set, action->node, "because node is unclean");
             }
 
         } else if (action->node->details->pending) {
@@ -699,7 +698,7 @@ unpack_operation(action_t * action, xmlNode * xml_obj, resource_t * container,
         }
     }
 
-    /* Begin compatability code */
+    /* Begin compatibility code ("requires" set on start action not resource) */
     value = g_hash_table_lookup(action->meta, "requires");
 
     if (safe_str_neq(action->task, RSC_START)
@@ -716,8 +715,8 @@ unpack_operation(action_t * action, xmlNode * xml_obj, resource_t * container,
     } else if (safe_str_eq(value, "unfencing")) {
         action->needs = rsc_req_stonith;
         set_bit(action->rsc->flags, pe_rsc_needs_unfencing);
-        if (is_set(data_set->flags, pe_flag_stonith_enabled)) {
-            crm_notice("%s requires (un)fencing but fencing is disabled", action->rsc->id);
+        if (is_not_set(data_set->flags, pe_flag_stonith_enabled)) {
+            crm_notice("%s requires unfencing but fencing is disabled", action->rsc->id);
         }
 
     } else if (is_set(data_set->flags, pe_flag_stonith_enabled)
@@ -726,7 +725,7 @@ unpack_operation(action_t * action, xmlNode * xml_obj, resource_t * container,
         if (is_not_set(data_set->flags, pe_flag_stonith_enabled)) {
             crm_notice("%s requires fencing but fencing is disabled", action->rsc->id);
         }
-        /* End compatability code */
+        /* End compatibility code */
 
     } else if (is_set(action->rsc->flags, pe_rsc_needs_fencing)) {
         action->needs = rsc_req_stonith;
@@ -803,7 +802,7 @@ unpack_operation(action_t * action, xmlNode * xml_obj, resource_t * container,
         action->on_fail = action_fail_restart_container;
         value = "restart container (and possibly migrate) (default)";
 
-    /* for barmetal remote nodes, ensure that any failure that results in
+    /* for baremetal remote nodes, ensure that any failure that results in
      * dropping an active connection to a remote node results in fencing of
      * the remote node.
      *
@@ -945,7 +944,6 @@ unpack_operation(action_t * action, xmlNode * xml_obj, resource_t * container,
     if (value_i < 0) {
         value_i = 0;
     }
-    value_i += start_delay;
     value_ms = crm_itoa(value_i);
     g_hash_table_replace(action->meta, strdup(field), value_ms);
 }
@@ -1069,7 +1067,7 @@ print_str_str(gpointer key, gpointer value, gpointer user_data)
 void
 print_resource(int log_level, const char *pre_text, resource_t * rsc, gboolean details)
 {
-    long options = pe_print_log;
+    long options = pe_print_log | pe_print_pending;
 
     if (rsc == NULL) {
         do_crm_log(log_level - 1, "%s%s: <NULL>",
@@ -1192,7 +1190,7 @@ find_first_action(GListPtr input, const char *uuid, const char *task, node_t * o
 }
 
 GListPtr
-find_actions(GListPtr input, const char *key, node_t * on_node)
+find_actions(GListPtr input, const char *key, const node_t *on_node)
 {
     GListPtr gIter = input;
     GListPtr result = NULL;
@@ -1202,24 +1200,29 @@ find_actions(GListPtr input, const char *key, node_t * on_node)
     for (; gIter != NULL; gIter = gIter->next) {
         action_t *action = (action_t *) gIter->data;
 
-        crm_trace("Matching %s against %s", key, action->uuid);
         if (safe_str_neq(key, action->uuid)) {
+            crm_trace("%s does not match action %s", key, action->uuid);
             continue;
 
         } else if (on_node == NULL) {
+            crm_trace("Action %s matches (ignoring node)", key);
             result = g_list_prepend(result, action);
 
         } else if (action->node == NULL) {
-            /* skip */
-            crm_trace("While looking for %s action on %s, "
-                      "found an unallocated one.  Assigning"
-                      " it to the requested node...", key, on_node->details->uname);
+            crm_trace("Action %s matches (unallocated, assigning to %s)",
+                      key, on_node->details->uname);
 
             action->node = node_copy(on_node);
             result = g_list_prepend(result, action);
 
         } else if (on_node->details == action->node->details) {
+            crm_trace("Action %s on %s matches", key, on_node->details->uname);
             result = g_list_prepend(result, action);
+
+        } else {
+            crm_trace("Action %s on node %s does not match requested node %s",
+                      key, action->node->details->uname,
+                      on_node->details->uname);
         }
     }
 
@@ -1291,18 +1294,18 @@ resource_location(resource_t * rsc, node_t * node, int score, const char *tag,
         GListPtr gIter = data_set->nodes;
 
         for (; gIter != NULL; gIter = gIter->next) {
-            node_t *node = (node_t *) gIter->data;
+            node_t *node_iter = (node_t *) gIter->data;
 
-            resource_node_score(rsc, node, score, tag);
+            resource_node_score(rsc, node_iter, score, tag);
         }
 
     } else {
         GHashTableIter iter;
-        node_t *node = NULL;
+        node_t *node_iter = NULL;
 
         g_hash_table_iter_init(&iter, rsc->allowed_nodes);
-        while (g_hash_table_iter_next(&iter, NULL, (void **)&node)) {
-            resource_node_score(rsc, node, score, tag);
+        while (g_hash_table_iter_next(&iter, NULL, (void **)&node_iter)) {
+            resource_node_score(rsc, node_iter, score, tag);
         }
     }
 
@@ -2020,85 +2023,6 @@ const char *rsc_printable_id(resource_t *rsc)
     return rsc->id;
 }
 
-gboolean
-is_rsc_baremetal_remote_node(resource_t *rsc, pe_working_set_t * data_set)
-{
-    node_t *node;
-
-    if (rsc == NULL) {
-        return FALSE;
-    } else if (rsc->is_remote_node == FALSE) {
-        return FALSE;
-    }
-
-    node = pe_find_node(data_set->nodes, rsc->id);
-    if (node == NULL) {
-        return FALSE;
-    }
-
-    return is_baremetal_remote_node(node);
-}
-
-gboolean
-is_baremetal_remote_node(node_t *node)
-{
-    if (is_remote_node(node) && (node->details->remote_rsc == FALSE || node->details->remote_rsc->container == FALSE)) {
-        return TRUE;
-    }
-    return FALSE;
-}
-
-gboolean
-is_container_remote_node(node_t *node)
-{
-    if (is_remote_node(node) && (node->details->remote_rsc && node->details->remote_rsc->container)) {
-        return TRUE;
-    }
-    return FALSE;
-}
-
-gboolean
-is_remote_node(node_t *node)
-{
-    if (node && node->details->type == node_remote) {
-        return TRUE;
-    }
-    return FALSE;
-}
-
-resource_t *
-rsc_contains_remote_node(pe_working_set_t * data_set, resource_t *rsc)
-{
-    if (is_set(data_set->flags, pe_flag_have_remote_nodes) == FALSE) {
-        return NULL;
-    }
-
-    if (rsc->fillers) {
-        GListPtr gIter = NULL;
-        for (gIter = rsc->fillers; gIter != NULL; gIter = gIter->next) {
-            resource_t *filler = (resource_t *) gIter->data;
-
-            if (filler->is_remote_node) {
-                return filler;
-            }
-        }
-    }
-    return NULL;
-}
-
-gboolean
-xml_contains_remote_node(xmlNode *xml)
-{
-    const char *class = crm_element_value(xml, XML_AGENT_ATTR_CLASS);
-    const char *provider = crm_element_value(xml, XML_AGENT_ATTR_PROVIDER);
-    const char *agent = crm_element_value(xml, XML_ATTR_TYPE);
-
-    if (safe_str_eq(agent, "remote") && safe_str_eq(provider, "pacemaker") && safe_str_eq(class, "ocf")) {
-        return TRUE;
-    }
-    return FALSE;
-}
-
 void
 clear_bit_recursive(resource_t * rsc, unsigned long long flag)
 {
