@@ -1,3 +1,10 @@
+/*
+ * Copyright (C) 2014-2016 Andrew Beekhof <andrew@beekhof.net>
+ *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
+ */
+
 #include <crm_internal.h>
 #include <crm/crm.h>
 #include <crm/services.h>
@@ -6,60 +13,69 @@
 
 #define BUS_PROPERTY_IFACE "org.freedesktop.DBus.Properties"
 
-struct db_getall_data
-{
-        char *name;
-        char *target;
-        char *object;
-        void *userdata;
-        void (*callback)(const char *name, const char *value, void *userdata);
+struct db_getall_data {
+    char *name;
+    char *target;
+    char *object;
+    void *userdata;
+    void (*callback)(const char *name, const char *value, void *userdata);
 };
 
-static bool pcmk_dbus_error_check(DBusError *err, const char *prefix, const char *function, int line) 
-{
-    if (err && dbus_error_is_set(err)) {
-        do_crm_log_alias(LOG_ERR, __FILE__, function, line, "%s: DBus error '%s'", prefix, err->message);
-        dbus_error_free(err);
-        return TRUE;
-    }
-    return FALSE;
-}
-
-DBusConnection *pcmk_dbus_connect(void)
+DBusConnection *
+pcmk_dbus_connect(void)
 {
     DBusError err;
     DBusConnection *connection;
 
     dbus_error_init(&err);
     connection = dbus_bus_get(DBUS_BUS_SYSTEM, &err);
-    if(pcmk_dbus_error_check(&err, "Could not connect to System DBus", __FUNCTION__, __LINE__)) {
+    if (dbus_error_is_set(&err)) {
+        crm_err("Could not connect to System DBus: %s", err.message);
+        dbus_error_free(&err);
         return NULL;
     }
-
     if(connection) {
         pcmk_dbus_connection_setup_with_select(connection);
     }
     return connection;
 }
 
-void pcmk_dbus_disconnect(DBusConnection *connection)
+void
+pcmk_dbus_disconnect(DBusConnection *connection)
 {
+    return;
 }
 
+/*!
+ * \internal
+ * \brief Check whether a DBus reply indicates an error occurred
+ *
+ * \param[in]  pending If non-NULL, indicates that a DBus request was sent
+ * \param[in]  reply   Reply received from DBus
+ * \param[out] ret     If non-NULL, will be set to DBus error, if any
+ *
+ * \return TRUE if an error was found, FALSE otherwise
+ *
+ * \note Following the DBus API convention, a TRUE return is exactly equivalent
+ *       to ret being set. If ret is provided and this function returns TRUE,
+ *       the caller is responsible for calling dbus_error_free() on ret when
+ *       done using it.
+ */
 bool
-pcmk_dbus_find_error(const char *method, DBusPendingCall* pending, DBusMessage *reply, DBusError *ret)
+pcmk_dbus_find_error(DBusPendingCall *pending, DBusMessage *reply,
+                     DBusError *ret)
 {
     DBusError error;
 
     dbus_error_init(&error);
 
     if(pending == NULL) {
-        error.name = "org.clusterlabs.pacemaker.NoRequest";
-        error.message = "No request sent";
+        dbus_set_error_const(&error, "org.clusterlabs.pacemaker.NoRequest",
+                             "No request sent");
 
     } else if(reply == NULL) {
-        error.name = "org.clusterlabs.pacemaker.NoReply";
-        error.message = "No reply";
+        dbus_set_error_const(&error, "org.clusterlabs.pacemaker.NoReply",
+                             "No reply");
 
     } else {
         DBusMessageIter args;
@@ -70,39 +86,42 @@ pcmk_dbus_find_error(const char *method, DBusPendingCall* pending, DBusMessage *
             case DBUS_MESSAGE_TYPE_METHOD_RETURN:
                 dbus_message_iter_init(reply, &args);
                 sig = dbus_message_iter_get_signature(&args);
-                crm_trace("Call to %s returned '%s'", method, sig);
+                crm_trace("DBus call returned output args '%s'", sig);
                 dbus_free(sig);
                 break;
             case DBUS_MESSAGE_TYPE_INVALID:
-                error.message = "Invalid reply";
-                error.name = "org.clusterlabs.pacemaker.InvalidReply";
-                crm_err("Error processing %s response: %s", method, error.message);
+                dbus_set_error_const(&error,
+                                     "org.clusterlabs.pacemaker.InvalidReply",
+                                     "Invalid reply");
                 break;
             case DBUS_MESSAGE_TYPE_METHOD_CALL:
-                error.message = "Invalid reply (method call)";
-                error.name = "org.clusterlabs.pacemaker.InvalidReply.Method";
-                crm_err("Error processing %s response: %s", method, error.message);
+                dbus_set_error_const(&error,
+                                     "org.clusterlabs.pacemaker.InvalidReply.Method",
+                                     "Invalid reply (method call)");
                 break;
             case DBUS_MESSAGE_TYPE_SIGNAL:
-                error.message = "Invalid reply (signal)";
-                error.name = "org.clusterlabs.pacemaker.InvalidReply.Signal";
-                crm_err("Error processing %s response: %s", method, error.message);
+                dbus_set_error_const(&error,
+                                     "org.clusterlabs.pacemaker.InvalidReply.Signal",
+                                     "Invalid reply (signal)");
                 break;
-
             case DBUS_MESSAGE_TYPE_ERROR:
-                dbus_set_error_from_message (&error, reply);
-                crm_info("%s error '%s': %s", method, error.name, error.message);
+                dbus_set_error_from_message(&error, reply);
                 break;
             default:
-                error.message = "Unknown reply type";
-                error.name = "org.clusterlabs.pacemaker.InvalidReply.Type";
-                crm_err("Error processing %s response: %s (%d)", method, error.message, dtype);
+                dbus_set_error(&error,
+                               "org.clusterlabs.pacemaker.InvalidReply.Type",
+                               "Unknown reply type %d", dtype);
         }
     }
 
-    if(error.name || error.message) {
+    if (dbus_error_is_set(&error)) {
+        crm_trace("DBus reply indicated error '%s' (%s)",
+                  error.name, error.message);
         if (ret) {
-            *ret = error;
+            dbus_error_init(ret);
+            dbus_move_error(&error, ret);
+        } else {
+            dbus_error_free(&error);
         }
         return TRUE;
     }
@@ -110,7 +129,24 @@ pcmk_dbus_find_error(const char *method, DBusPendingCall* pending, DBusMessage *
     return FALSE;
 }
 
-DBusMessage *pcmk_dbus_send_recv(DBusMessage *msg, DBusConnection *connection, DBusError *error, int timeout)
+/*!
+ * \internal
+ * \brief Send a DBus request and wait for the reply
+ *
+ * \param[in]  msg         DBus request to send
+ * \param[in]  connection  DBus connection to use
+ * \param[out] error       If non-NULL, will be set to error, if any
+ * \param[in]  timeout     Timeout to use for request
+ *
+ * \return DBus reply
+ *
+ * \note If error is non-NULL, it is initialized, so the caller may always use
+ *       dbus_error_is_set() to determine whether an error occurred; the caller
+ *       is responsible for calling dbus_error_free() in this case.
+ */
+DBusMessage *
+pcmk_dbus_send_recv(DBusMessage *msg, DBusConnection *connection,
+                    DBusError *error, int timeout)
 {
     const char *method = NULL;
     DBusMessage *reply = NULL;
@@ -119,18 +155,22 @@ DBusMessage *pcmk_dbus_send_recv(DBusMessage *msg, DBusConnection *connection, D
     CRM_ASSERT(dbus_message_get_type (msg) == DBUS_MESSAGE_TYPE_METHOD_CALL);
     method = dbus_message_get_member (msg);
 
+    /* Ensure caller can reliably check whether error is set */
+    if (error) {
+        dbus_error_init(error);
+    }
+
     if (timeout <= 0) {
+        /* DBUS_TIMEOUT_USE_DEFAULT (-1) tells DBus to use a sane default */
         timeout = DBUS_TIMEOUT_USE_DEFAULT;
     }
 
     // send message and get a handle for a reply
-    if (!dbus_connection_send_with_reply (connection, msg, &pending, timeout/* -1 is default timeout, aka. DBUS_TIMEOUT_USE_DEFAULT */)) {
+    if (!dbus_connection_send_with_reply(connection, msg, &pending, timeout)) {
         if(error) {
-            dbus_error_init(error);
-            error->message = "Call to dbus_connection_send_with_reply() failed";
-            error->name = "org.clusterlabs.pacemaker.SendFailed";
+            dbus_set_error(error, "org.clusterlabs.pacemaker.SendFailed",
+                           "Could not queue DBus '%s' request", method);
         }
-        crm_err("Error sending %s request", method);
         return NULL;
     }
 
@@ -144,7 +184,7 @@ DBusMessage *pcmk_dbus_send_recv(DBusMessage *msg, DBusConnection *connection, D
         reply = dbus_pending_call_steal_reply(pending);
     }
 
-    (void)pcmk_dbus_find_error(method, pending, reply, error);
+    (void)pcmk_dbus_find_error(pending, reply, error);
 
     if(pending) {
         /* free the pending message handle */
@@ -154,26 +194,26 @@ DBusMessage *pcmk_dbus_send_recv(DBusMessage *msg, DBusConnection *connection, D
     return reply;
 }
 
-DBusPendingCall* pcmk_dbus_send(DBusMessage *msg, DBusConnection *connection,
-                    void(*done)(DBusPendingCall *pending, void *user_data), void *user_data, int timeout)
+DBusPendingCall *
+pcmk_dbus_send(DBusMessage *msg, DBusConnection *connection,
+               void(*done)(DBusPendingCall *pending, void *user_data),
+               void *user_data, int timeout)
 {
-    DBusError error;
     const char *method = NULL;
     DBusPendingCall* pending = NULL;
 
-    dbus_error_init(&error);
-
     CRM_ASSERT(done);
     CRM_ASSERT(dbus_message_get_type (msg) == DBUS_MESSAGE_TYPE_METHOD_CALL);
     method = dbus_message_get_member (msg);
 
 
     if (timeout <= 0) {
+        /* DBUS_TIMEOUT_USE_DEFAULT (-1) tells DBus to use a sane default */
         timeout = DBUS_TIMEOUT_USE_DEFAULT;
     }
 
     // send message and get a handle for a reply
-    if (!dbus_connection_send_with_reply (connection, msg, &pending, timeout/* -1 is default timeout, aka. DBUS_TIMEOUT_USE_DEFAULT */)) {
+    if (!dbus_connection_send_with_reply(connection, msg, &pending, timeout)) {
         crm_err("Send with reply failed for %s", method);
         return NULL;
 
@@ -187,11 +227,11 @@ DBusPendingCall* pcmk_dbus_send(DBusMessage *msg, DBusConnection *connection,
         crm_info("DBus %s call completed too soon", method);
         if(done) {
 #if 0
-        /* This sounds like a good idea, but allegedly it breaks things */
-        done(pending, user_data);
-        pending = NULL;
+            /* This sounds like a good idea, but allegedly it breaks things */
+            done(pending, user_data);
+            pending = NULL;
 #else
-        CRM_ASSERT(dbus_pending_call_set_notify(pending, done, user_data, NULL));
+            CRM_ASSERT(dbus_pending_call_set_notify(pending, done, user_data, NULL));
 #endif
         }
 
@@ -201,7 +241,9 @@ DBusPendingCall* pcmk_dbus_send(DBusMessage *msg, DBusConnection *connection,
     return pending;
 }
 
-bool pcmk_dbus_type_check(DBusMessage *msg, DBusMessageIter *field, int expected, const char *function, int line)
+bool
+pcmk_dbus_type_check(DBusMessage *msg, DBusMessageIter *field, int expected,
+                     const char *function, int line)
 {
     int dtype = 0;
     DBusMessageIter lfield;
@@ -244,8 +286,10 @@ pcmk_dbus_lookup_result(DBusMessage *reply, struct db_getall_data *data)
     DBusMessageIter dict;
     DBusMessageIter args;
 
-    if(pcmk_dbus_find_error("GetAll", (void*)&error, reply, &error)) {
-        crm_err("Cannot get properties from %s for %s", data->target, data->object);
+    if (pcmk_dbus_find_error((void*)&error, reply, &error)) {
+        crm_err("Cannot get properties from %s for %s: %s",
+                data->target, data->object, error.message);
+        dbus_error_free(&error);
         goto cleanup;
     }
 
@@ -340,25 +384,21 @@ pcmk_dbus_lookup_cb(DBusPendingCall *pending, void *user_data)
 }
 
 char *
-pcmk_dbus_get_property(
-    DBusConnection *connection, const char *target, const char *obj, const gchar * iface, const char *name,
-    void (*callback)(const char *name, const char *value, void *userdata), void *userdata, DBusPendingCall **pending,
-    int timeout)
+pcmk_dbus_get_property(DBusConnection *connection, const char *target,
+                       const char *obj, const gchar * iface, const char *name,
+                       void (*callback)(const char *name, const char *value, void *userdata),
+                       void *userdata, DBusPendingCall **pending, int timeout)
 {
     DBusMessage *msg;
     const char *method = "GetAll";
     char *output = NULL;
-
     struct db_getall_data *query_data = NULL;
 
-    /* char *state = pcmk_dbus_get_property(systemd_proxy, BUS_NAME, unit, BUS_NAME ".Unit", "ActiveState"); */
-
     crm_debug("Calling: %s on %s", method, target);
     msg = dbus_message_new_method_call(target, // target for the method call
                                        obj, // object to call on
                                        BUS_PROPERTY_IFACE, // interface to call on
                                        method); // method name
-
     if (NULL == msg) {
         crm_err("Call to %s failed: No message", method);
         return NULL;
@@ -404,7 +444,10 @@ pcmk_dbus_get_property(
     return output;
 }
 
-static void pcmk_dbus_connection_dispatch(DBusConnection *connection, DBusDispatchStatus new_status, void *data){
+static void
+pcmk_dbus_connection_dispatch(DBusConnection *connection,
+                              DBusDispatchStatus new_status, void *data)
+{
     crm_trace("status %d for %p", new_status, data);
     if (new_status == DBUS_DISPATCH_DATA_REMAINS){
         dbus_connection_dispatch(connection);
@@ -418,20 +461,20 @@ static void pcmk_dbus_connection_dispatch(DBusConnection *connection, DBusDispat
 /* Copied from dbus-watch.c */
 
 static const char*
-dbus_watch_flags_to_string (int flags)
+dbus_watch_flags_to_string(int flags)
 {
-  const char *watch_type;
+    const char *watch_type;
 
-  if ((flags & DBUS_WATCH_READABLE) &&
-      (flags & DBUS_WATCH_WRITABLE))
-    watch_type = "readwrite";
-  else if (flags & DBUS_WATCH_READABLE)
-    watch_type = "read";
-  else if (flags & DBUS_WATCH_WRITABLE)
-    watch_type = "write";
-  else
-    watch_type = "not read or write";
-  return watch_type;
+    if ((flags & DBUS_WATCH_READABLE) && (flags & DBUS_WATCH_WRITABLE)) {
+        watch_type = "readwrite";
+    } else if (flags & DBUS_WATCH_READABLE) {
+        watch_type = "read";
+    } else if (flags & DBUS_WATCH_WRITABLE) {
+        watch_type = "write";
+    } else {
+        watch_type = "not read or write";
+    }
+    return watch_type;
 }
 
 static int
@@ -456,11 +499,13 @@ pcmk_dbus_watch_dispatch(gpointer userdata)
 
     if(flags != dbus_watch_get_flags(watch)) {
         flags = dbus_watch_get_flags(watch);
-        crm_trace("Dispatched client %p: %s (%d)", client, dbus_watch_flags_to_string(flags), flags);
+        crm_trace("Dispatched client %p: %s (%d)", client,
+                  dbus_watch_flags_to_string(flags), flags);
     }
 
     if(oom) {
-        crm_err("DBus encountered OOM while attempting to dispatch %p (%s)", client, dbus_watch_flags_to_string(flags));
+        crm_err("DBus encountered OOM while attempting to dispatch %p (%s)",
+                client, dbus_watch_flags_to_string(flags));
     }
     return 0;
 }
@@ -479,7 +524,8 @@ struct mainloop_fd_callbacks pcmk_dbus_cb = {
 };
 
 static dbus_bool_t
-pcmk_dbus_watch_add(DBusWatch *watch, void *data){
+pcmk_dbus_watch_add(DBusWatch *watch, void *data)
+{
     int fd = dbus_watch_get_unix_fd(watch);
 
     mainloop_io_t *client = mainloop_add_fd(
@@ -494,12 +540,14 @@ static void
 pcmk_dbus_watch_toggle(DBusWatch *watch, void *data)
 {
     mainloop_io_t *client = dbus_watch_get_data(watch);
-    crm_notice("DBus client %p is now %s", client, dbus_watch_get_enabled(watch)?"enabled":"disabled");
+    crm_notice("DBus client %p is now %s",
+               client, (dbus_watch_get_enabled(watch)? "enabled" : "disabled"));
 }
 
 
 static void
-pcmk_dbus_watch_remove(DBusWatch *watch, void *data){
+pcmk_dbus_watch_remove(DBusWatch *watch, void *data)
+{
     mainloop_io_t *client = dbus_watch_get_data(watch);
 
     crm_trace("Removed client %p (%p)", client, data);
@@ -515,10 +563,12 @@ pcmk_dbus_timeout_dispatch(gpointer data)
 }
 
 static dbus_bool_t
-pcmk_dbus_timeout_add(DBusTimeout *timeout, void *data){
-    guint id = g_timeout_add(dbus_timeout_get_interval(timeout), pcmk_dbus_timeout_dispatch, timeout);
+pcmk_dbus_timeout_add(DBusTimeout *timeout, void *data)
+{
+    guint id = g_timeout_add(dbus_timeout_get_interval(timeout),
+                             pcmk_dbus_timeout_dispatch, timeout);
 
-    crm_trace("Adding timeout %p (%ld)", timeout, dbus_timeout_get_interval(timeout));
+    crm_trace("Adding timeout %p (%d)", timeout, dbus_timeout_get_interval(timeout));
 
     if(id) {
         dbus_timeout_set_data(timeout, GUINT_TO_POINTER(id), NULL);
@@ -527,7 +577,8 @@ pcmk_dbus_timeout_add(DBusTimeout *timeout, void *data){
 }
 
 static void
-pcmk_dbus_timeout_remove(DBusTimeout *timeout, void *data){
+pcmk_dbus_timeout_remove(DBusTimeout *timeout, void *data)
+{
     void *vid = dbus_timeout_get_data(timeout);
     guint id = GPOINTER_TO_UINT(vid);
 
@@ -540,7 +591,8 @@ pcmk_dbus_timeout_remove(DBusTimeout *timeout, void *data){
 }
 
 static void
-pcmk_dbus_timeout_toggle(DBusTimeout *timeout, void *data){
+pcmk_dbus_timeout_toggle(DBusTimeout *timeout, void *data)
+{
     bool enabled = dbus_timeout_get_enabled(timeout);
 
     crm_trace("Toggling timeout for %p to %s", timeout, enabled?"off":"on");
@@ -554,12 +606,16 @@ pcmk_dbus_timeout_toggle(DBusTimeout *timeout, void *data){
 
 /* Inspired by http://www.kolej.mff.cuni.cz/~vesej3am/devel/dbus-select.c */
 
-void pcmk_dbus_connection_setup_with_select(DBusConnection *c){
-        dbus_connection_set_exit_on_disconnect (c, FALSE);
-	dbus_connection_set_timeout_functions(
-            c, pcmk_dbus_timeout_add, pcmk_dbus_timeout_remove, pcmk_dbus_timeout_toggle, NULL, NULL);
-	dbus_connection_set_watch_functions(c, pcmk_dbus_watch_add, pcmk_dbus_watch_remove, pcmk_dbus_watch_toggle, NULL, NULL);
-	dbus_connection_set_dispatch_status_function(c, pcmk_dbus_connection_dispatch, NULL, NULL);
-
-	pcmk_dbus_connection_dispatch(c, dbus_connection_get_dispatch_status(c), NULL);
+void
+pcmk_dbus_connection_setup_with_select(DBusConnection *c)
+{
+    dbus_connection_set_exit_on_disconnect(c, FALSE);
+    dbus_connection_set_timeout_functions(c, pcmk_dbus_timeout_add,
+                                          pcmk_dbus_timeout_remove,
+                                          pcmk_dbus_timeout_toggle, NULL, NULL);
+    dbus_connection_set_watch_functions(c, pcmk_dbus_watch_add,
+                                        pcmk_dbus_watch_remove,
+                                        pcmk_dbus_watch_toggle, NULL, NULL);
+    dbus_connection_set_dispatch_status_function(c, pcmk_dbus_connection_dispatch, NULL, NULL);
+    pcmk_dbus_connection_dispatch(c, dbus_connection_get_dispatch_status(c), NULL);
 }
