@@ -1,19 +1,8 @@
 /*
- * Copyright (C) 2010 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright (C) 2010-2016 Andrew Beekhof <andrew@beekhof.net>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -89,7 +78,7 @@ svc_read_output(int fd, svc_action_t * op, bool is_stderr)
         crm_trace("Reading %s stdout into offset %d", op->id, len);
 
     } else {
-        crm_trace("Reading %s %s", op->id, is_stderr?"stderr":"stdout", len);
+        crm_trace("Reading %s %s into offset %d", op->id, is_stderr?"stderr":"stdout", len);
     }
 
     do {
@@ -457,13 +446,21 @@ action_launch_child(svc_action_t *op)
     _exit(op->rc);
 }
 
-static void
-action_synced_wait(svc_action_t * op, sigset_t mask)
-{
-
 #ifndef HAVE_SYS_SIGNALFD_H
-    CRM_ASSERT(FALSE);
-#else
+static int sigchld_pipe[2] = { -1, -1 };
+
+static void
+sigchld_handler()
+{
+    if ((sigchld_pipe[1] >= 0) && (write(sigchld_pipe[1], "", 1) == -1)) {
+        crm_perror(LOG_TRACE, "Could not poke SIGCHLD self-pipe");
+    }
+}
+#endif
+
+static void
+action_synced_wait(svc_action_t * op, sigset_t *mask)
+{
     int status = 0;
     int timeout = op->timeout;
     int sfd = -1;
@@ -471,10 +468,14 @@ action_synced_wait(svc_action_t * op, sigset_t mask)
     struct pollfd fds[3];
     int wait_rc = 0;
 
-    sfd = signalfd(-1, &mask, SFD_NONBLOCK);
+#ifdef HAVE_SYS_SIGNALFD_H
+    sfd = signalfd(-1, mask, SFD_NONBLOCK);
     if (sfd < 0) {
         crm_perror(LOG_ERR, "signalfd() failed");
     }
+#else
+    sfd = sigchld_pipe[0];
+#endif
 
     fds[0].fd = op->opaque->stdout_fd;
     fds[0].events = POLLIN;
@@ -503,6 +504,7 @@ action_synced_wait(svc_action_t * op, sigset_t mask)
             }
 
             if (fds[2].revents & POLLIN) {
+#ifdef HAVE_SYS_SIGNALFD_H
                 struct signalfd_siginfo fdsi;
                 ssize_t s;
 
@@ -511,6 +513,12 @@ action_synced_wait(svc_action_t * op, sigset_t mask)
                     crm_perror(LOG_ERR, "Read from signal fd %d failed", sfd);
 
                 } else if (fdsi.ssi_signo == SIGCHLD) {
+#else
+                if (1) {
+                    /* Clear out the sigchld pipe. */
+                    char ch;
+                    while (read(sfd, &ch, 1) == 1);
+#endif
                     wait_rc = waitpid(op->pid, &status, WNOHANG);
 
                     if (wait_rc < 0){
@@ -583,10 +591,10 @@ action_synced_wait(svc_action_t * op, sigset_t mask)
 
     close(op->opaque->stdout_fd);
     close(op->opaque->stderr_fd);
+
+#ifdef HAVE_SYS_SIGNALFD_H
     close(sfd);
-
 #endif
-
 }
 
 /* For an asynchronous 'op', returns FALSE if 'op' should be free'd by the caller */
@@ -596,17 +604,31 @@ services_os_action_execute(svc_action_t * op, gboolean synchronous)
 {
     int stdout_fd[2];
     int stderr_fd[2];
+    struct stat st;
+    sigset_t *pmask;
+
+#ifdef HAVE_SYS_SIGNALFD_H
     sigset_t mask;
     sigset_t old_mask;
-    struct stat st;
-
-    if (pipe(stdout_fd) < 0) {
-        crm_err("pipe() failed");
-    }
-
-    if (pipe(stderr_fd) < 0) {
-        crm_err("pipe() failed");
-    }
+#define sigchld_cleanup() do {                                                \
+    if (sigismember(&old_mask, SIGCHLD) == 0) {                               \
+        if (sigprocmask(SIG_UNBLOCK, &mask, NULL) < 0) {                      \
+            crm_perror(LOG_ERR, "sigprocmask() failed to unblock sigchld");   \
+        }                                                                     \
+    }                                                                         \
+} while (0)
+#else
+    struct sigaction sa;
+    struct sigaction old_sa;
+#define sigchld_cleanup() do {                                                \
+    if (sigaction(SIGCHLD, &old_sa, NULL) < 0) {                              \
+        crm_perror(LOG_ERR, "sigaction() failed to remove sigchld handler");  \
+    }                                                                         \
+    close(sigchld_pipe[0]);                                                   \
+    close(sigchld_pipe[1]);                                                   \
+    sigchld_pipe[0] = sigchld_pipe[1] = -1;                                   \
+} while(0)
+#endif
 
     /* Fail fast */
     if(stat(op->opaque->exec, &st) != 0) {
@@ -619,14 +641,61 @@ services_os_action_execute(svc_action_t * op, gboolean synchronous)
         return FALSE;
     }
 
+    if (pipe(stdout_fd) < 0) {
+        int rc = errno;
+
+        crm_err("pipe(stdout_fd) failed. '%s': %s (%d)", op->opaque->exec, pcmk_strerror(rc), rc);
+
+        services_handle_exec_error(op, rc);
+        if (!synchronous) {
+            return operation_finalize(op);
+        }
+        return FALSE;
+    }
+
+    if (pipe(stderr_fd) < 0) {
+        int rc = errno;
+
+        close(stdout_fd[0]);
+        close(stdout_fd[1]);
+
+        crm_err("pipe(stderr_fd) failed. '%s': %s (%d)", op->opaque->exec, pcmk_strerror(rc), rc);
+
+        services_handle_exec_error(op, rc);
+        if (!synchronous) {
+            return operation_finalize(op);
+        }
+        return FALSE;
+    }
+
     if (synchronous) {
+#ifdef HAVE_SYS_SIGNALFD_H
         sigemptyset(&mask);
         sigaddset(&mask, SIGCHLD);
         sigemptyset(&old_mask);
 
         if (sigprocmask(SIG_BLOCK, &mask, &old_mask) < 0) {
-            crm_perror(LOG_ERR, "sigprocmask() failed");
+            crm_perror(LOG_ERR, "sigprocmask() failed to block sigchld");
         }
+
+        pmask = &mask;
+#else
+        if(pipe(sigchld_pipe) == -1) {
+            crm_perror(LOG_ERR, "pipe() failed");
+        }
+
+        set_fd_opts(sigchld_pipe[0], O_NONBLOCK);
+        set_fd_opts(sigchld_pipe[1], O_NONBLOCK);
+
+        sa.sa_handler = sigchld_handler;
+        sa.sa_flags = 0;
+        sigemptyset(&sa.sa_mask);
+        if (sigaction(SIGCHLD, &sa, &old_sa) < 0) {
+            crm_perror(LOG_ERR, "sigaction() failed to set sigchld handler");
+        }
+
+        pmask = NULL;
+#endif
     }
 
     op->pid = fork();
@@ -645,6 +714,8 @@ services_os_action_execute(svc_action_t * op, gboolean synchronous)
                 if (!synchronous) {
                     return operation_finalize(op);
                 }
+
+                sigchld_cleanup();
                 return FALSE;
             }
         case 0:                /* Child */
@@ -663,7 +734,12 @@ services_os_action_execute(svc_action_t * op, gboolean synchronous)
                 close(stderr_fd[1]);
             }
 
+            if (synchronous) {
+                sigchld_cleanup();
+            }
+
             action_launch_child(op);
+            CRM_ASSERT(0);  /* action_launch_child is effectively noreturn */
     }
 
     /* Only the parent reaches here */
@@ -677,14 +753,8 @@ services_os_action_execute(svc_action_t * op, gboolean synchronous)
     set_fd_opts(op->opaque->stderr_fd, O_NONBLOCK);
 
     if (synchronous) {
-        action_synced_wait(op, mask);
-
-        if (sigismember(&old_mask, SIGCHLD) == 0) {
-            if (sigprocmask(SIG_UNBLOCK, &mask, NULL) < 0) {
-                crm_perror(LOG_ERR, "sigprocmask() to unblocked failed");
-            }
-        }
-
+        action_synced_wait(op, pmask);
+        sigchld_cleanup();
     } else {
 
         crm_trace("Async waiting for %d - %s", op->pid, op->opaque->exec);
