@@ -41,7 +41,11 @@ unpack_action(synapse_t * parent, xmlNode * xml_action)
     }
 
     action = calloc(1, sizeof(crm_action_t));
-    CRM_CHECK(action != NULL, return NULL);
+    if (action == NULL) {
+        crm_perror(LOG_CRIT, "Cannot unpack action");
+        crm_log_xml_trace(xml_action, "Lost action");
+        return NULL;
+    }
 
     action->id = crm_parse_int(value, NULL);
     action->type = action_type_rsc;
