@@ -541,7 +541,7 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
         if (client) {
             send_client_notify(client->id, client, notify);
         }
-    } else {
+    } else if (client_connections != NULL) {
         g_hash_table_foreach(client_connections, send_client_notify, notify);
     }
 
@@ -551,24 +551,26 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
 static void
 send_generic_notify(int rc, xmlNode * request)
 {
-    int call_id = 0;
-    xmlNode *notify = NULL;
-    xmlNode *rsc_xml = get_xpath_object("//" F_LRMD_RSC, request, LOG_ERR);
-    const char *rsc_id = crm_element_value(rsc_xml, F_LRMD_RSC_ID);
-    const char *op = crm_element_value(request, F_LRMD_OPERATION);
+    if (client_connections != NULL) {
+        int call_id = 0;
+        xmlNode *notify = NULL;
+        xmlNode *rsc_xml = get_xpath_object("//" F_LRMD_RSC, request, LOG_ERR);
+        const char *rsc_id = crm_element_value(rsc_xml, F_LRMD_RSC_ID);
+        const char *op = crm_element_value(request, F_LRMD_OPERATION);
 
-    crm_element_value_int(request, F_LRMD_CALLID, &call_id);
+        crm_element_value_int(request, F_LRMD_CALLID, &call_id);
 
-    notify = create_xml_node(NULL, T_LRMD_NOTIFY);
-    crm_xml_add(notify, F_LRMD_ORIGIN, __FUNCTION__);
-    crm_xml_add_int(notify, F_LRMD_RC, rc);
-    crm_xml_add_int(notify, F_LRMD_CALLID, call_id);
-    crm_xml_add(notify, F_LRMD_OPERATION, op);
-    crm_xml_add(notify, F_LRMD_RSC_ID, rsc_id);
+        notify = create_xml_node(NULL, T_LRMD_NOTIFY);
+        crm_xml_add(notify, F_LRMD_ORIGIN, __FUNCTION__);
+        crm_xml_add_int(notify, F_LRMD_RC, rc);
+        crm_xml_add_int(notify, F_LRMD_CALLID, call_id);
+        crm_xml_add(notify, F_LRMD_OPERATION, op);
+        crm_xml_add(notify, F_LRMD_RSC_ID, rsc_id);
 
-    g_hash_table_foreach(client_connections, send_client_notify, notify);
+        g_hash_table_foreach(client_connections, send_client_notify, notify);
 
-    free_xml(notify);
+        free_xml(notify);
+    }
 }
 
 static void
@@ -885,7 +887,7 @@ action_complete(svc_action_t * action)
              *
              * So we have to jump through a few hoops so that we don't
              * report 'complete' to the rest of pacemaker until, you know,
-             * its actually done.
+             * it's actually done.
              */
             goagain = true;
             cmd->real_action = cmd->action;
@@ -900,6 +902,8 @@ action_complete(svc_action_t * action)
             /* Ok, so this is the follow up monitor action to check if start actually completed */
             if(cmd->lrmd_op_status == PCMK_LRM_OP_DONE && cmd->exec_rc == PCMK_OCF_PENDING) {
                 goagain = true;
+            } else if(cmd->exec_rc == PCMK_OCF_OK && safe_str_eq(cmd->real_action, "stop")) {
+                goagain = true;
 
             } else {
 #ifdef HAVE_SYS_TIMEB_H
@@ -1378,13 +1382,16 @@ process_lrmd_signon(crm_client_t * client, uint32_t id, xmlNode * request)
     const char *is_ipc_provider = crm_element_value(request, F_LRMD_IS_IPC_PROVIDER);
     const char *protocol_version = crm_element_value(request, F_LRMD_PROTOCOL_VERSION);
 
-    if (safe_str_neq(protocol_version, LRMD_PROTOCOL_VERSION)) {
+    if (compare_version(protocol_version, LRMD_PROTOCOL_VERSION) < 0) {
+        crm_err("Cluster API version must be greater than or equal to %s, not %s",
+                LRMD_PROTOCOL_VERSION, protocol_version);
         crm_xml_add_int(reply, F_LRMD_RC, -EPROTO);
         crm_xml_add(reply, F_LRMD_PROTOCOL_VERSION, LRMD_PROTOCOL_VERSION);
     }
 
     crm_xml_add(reply, F_LRMD_OPERATION, CRM_OP_REGISTER);
     crm_xml_add(reply, F_LRMD_CLIENTID, client->id);
+    crm_xml_add(reply, F_LRMD_PROTOCOL_VERSION, LRMD_PROTOCOL_VERSION);
     lrmd_server_send_reply(client, id, reply);
 
     if (crm_is_true(is_ipc_provider)) {
@@ -1538,7 +1545,7 @@ cancel_op(const char *rsc_id, const char *action, int interval)
      * 1. Check pending ops list, if it hasn't been handed off
      *    to the service library or stonith recurring list remove
      *    it there and that will stop it.
-     * 2. If it isn't in the pending ops list, then its either a
+     * 2. If it isn't in the pending ops list, then it's either a
      *    recurring op in the stonith recurring list, or the service
      *    library's recurring list.  Stop it there
      * 3. If not found in any lists, then this operation has either
@@ -1561,7 +1568,7 @@ cancel_op(const char *rsc_id, const char *action, int interval)
 
     if (safe_str_eq(rsc->class, "stonith")) {
         /* The service library does not handle stonith operations.
-         * We have to handle recurring stonith opereations ourselves. */
+         * We have to handle recurring stonith operations ourselves. */
         for (gIter = rsc->recurring_ops; gIter != NULL; gIter = gIter->next) {
             lrmd_cmd_t *cmd = gIter->data;
 
@@ -1681,6 +1688,11 @@ process_lrmd_message(crm_client_t * client, uint32_t id, xmlNode * request)
     } else if (crm_str_eq(op, LRMD_OP_POKE, TRUE)) {
         do_notify = 1;
         do_reply = 1;
+    } else if (crm_str_eq(op, LRMD_OP_CHECK, TRUE)) {
+        xmlNode *data = get_message_xml(request, F_LRMD_CALLDATA); 
+        const char *timeout = crm_element_value(data, F_LRMD_WATCHDOG);
+        CRM_LOG_ASSERT(data != NULL);
+        check_sbd_timeout(timeout);
     } else {
         rc = -EOPNOTSUPP;
         do_reply = 1;
@@ -1688,8 +1700,8 @@ process_lrmd_message(crm_client_t * client, uint32_t id, xmlNode * request)
         crm_log_xml_warn(request, "UnknownOp");
     }
 
-    crm_debug("Processed %s operation from %s: rc=%d, reply=%d, notify=%d, exit=%d",
-              op, client->id, rc, do_reply, do_notify, exit);
+    crm_debug("Processed %s operation from %s: rc=%d, reply=%d, notify=%d",
+              op, client->id, rc, do_reply, do_notify);
 
     if (do_reply) {
         send_reply(client, rc, id, call_id);
