@@ -20,6 +20,7 @@
 
 #include <crm/msg_xml.h>
 #include <allocate.h>
+#include <notif.h>
 #include <utils.h>
 #include <allocate.h>
 
@@ -845,12 +846,12 @@ clone_create_actions(resource_t * rsc, pe_working_set_t * data_set)
     started = custom_action(rsc, started_key(rsc),
                             RSC_STARTED, NULL, !child_starting, TRUE, data_set);
 
-    update_action_flags(start, pe_action_pseudo | pe_action_runnable);
-    update_action_flags(started, pe_action_pseudo);
+    update_action_flags(start, pe_action_pseudo | pe_action_runnable, __FUNCTION__);
+    update_action_flags(started, pe_action_pseudo, __FUNCTION__);
     started->priority = INFINITY;
 
     if (child_active || child_starting) {
-        update_action_flags(started, pe_action_runnable);
+        update_action_flags(started, pe_action_runnable, __FUNCTION__);
     }
 
     child_ordering_constraints(rsc, data_set);
@@ -865,11 +866,11 @@ clone_create_actions(resource_t * rsc, pe_working_set_t * data_set)
                             RSC_STOPPED, NULL, !child_stopping, TRUE, data_set);
 
     stopped->priority = INFINITY;
-    update_action_flags(stop, pe_action_pseudo | pe_action_runnable);
+    update_action_flags(stop, pe_action_pseudo | pe_action_runnable, __FUNCTION__);
     if (allow_dependent_migrations) {
-        update_action_flags(stop, pe_action_migrate_runnable);
+        update_action_flags(stop, pe_action_migrate_runnable, __FUNCTION__);
     }
-    update_action_flags(stopped, pe_action_pseudo | pe_action_runnable);
+    update_action_flags(stopped, pe_action_pseudo | pe_action_runnable, __FUNCTION__);
     if (clone_data->stop_notify == NULL) {
         clone_data->stop_notify =
             create_notification_boundaries(rsc, RSC_STOP, stop, stopped, data_set);
@@ -927,9 +928,11 @@ clone_internal_constraints(resource_t * rsc, pe_working_set_t * data_set)
     }
 }
 
-static void
+static bool
 assign_node(resource_t * rsc, node_t * node, gboolean force)
 {
+    bool changed = FALSE;
+
     if (rsc->children) {
 
         GListPtr gIter = rsc->children;
@@ -937,12 +940,17 @@ assign_node(resource_t * rsc, node_t * node, gboolean force)
         for (; gIter != NULL; gIter = gIter->next) {
             resource_t *child_rsc = (resource_t *) gIter->data;
 
-            native_assign_node(child_rsc, NULL, node, force);
+            changed |= native_assign_node(child_rsc, NULL, node, force);
         }
 
-        return;
+        return changed;
     }
+    if (rsc->allocated_to != NULL) {
+        changed = true;
+    }
+
     native_assign_node(rsc, NULL, node, force);
+    return changed;
 }
 
 static resource_t *
@@ -1113,6 +1121,7 @@ clone_rsc_colocation_rh(resource_t * rsc_lh, resource_t * rsc_rh, rsc_colocation
             node_t *chosen = child_rsc->fns->location(child_rsc, NULL, FALSE);
 
             if (chosen != NULL && is_set_recursive(child_rsc, pe_rsc_block, TRUE) == FALSE) {
+                pe_rsc_trace(rsc_rh, "Allowing %s: %s %d", constraint->id, chosen->details->uname, chosen->weight);
                 rhs = g_list_prepend(rhs, chosen);
             }
         }
@@ -1186,14 +1195,14 @@ clone_action_flags(action_t * action, node_t * node)
 
         child_action =
             find_first_action(child->actions, NULL, task_s, child->children ? NULL : node);
-        pe_rsc_trace(child, "Checking for %s in %s on %s", task_s, child->id,
+        pe_rsc_trace(action->rsc, "Checking for %s in %s on %s", task_s, child->id,
                      node ? node->details->uname : "none");
         if (child_action) {
             enum pe_action_flags child_flags = child->cmds->action_flags(child_action, node);
 
             if (is_set(flags, pe_action_optional)
                 && is_set(child_flags, pe_action_optional) == FALSE) {
-                pe_rsc_trace(child, "%s is manditory because of %s", action->uuid,
+                pe_rsc_trace(child, "%s is mandatory because of %s", action->uuid,
                              child_action->uuid);
                 flags = crm_clear_bit(__FUNCTION__, action->rsc->id, flags, pe_action_optional);
                 pe_clear_action_bit(action, pe_action_optional);
@@ -1240,7 +1249,8 @@ clone_update_actions_interleave(action_t * first, action_t * then, node_t * node
     const char *first_task = task2text(task);
 
     /* Fix this - lazy */
-    if (strstr(first->uuid, "_stopped_0") || strstr(first->uuid, "_demoted_0")) {
+    if (crm_ends_with(first->uuid, "_stopped_0")
+        || crm_ends_with(first->uuid, "_demoted_0")) {
         current = TRUE;
     }
 
@@ -1264,8 +1274,9 @@ clone_update_actions_interleave(action_t * first, action_t * then, node_t * node
              */
             if (type & (pe_order_runnable_left | pe_order_implies_then) /* Mandatory */ ) {
                 pe_rsc_info(then->rsc, "Inhibiting %s from being active", then_child->id);
-                assign_node(then_child, NULL, TRUE);
-                /* TODO - set changed correctly? */
+                if(assign_node(then_child, NULL, TRUE)) {
+                    changed |= pe_graph_updated_then;
+                }
             }
 
         } else {
@@ -1277,19 +1288,37 @@ clone_update_actions_interleave(action_t * first, action_t * then, node_t * node
             first_action = find_first_action(first_child->actions, NULL, first_task, node);
             then_action = find_first_action(then_child->actions, NULL, then->task, node);
 
-            CRM_CHECK(first_action != NULL || is_set(first_child->flags, pe_rsc_orphan),
-                      crm_err("No action found for %s in %s (first)", first_task, first_child->id));
+            if (first_action == NULL) {
+                if (is_not_set(first_child->flags, pe_rsc_orphan)
+                    && crm_str_eq(first_task, RSC_STOP, TRUE) == FALSE
+                    && crm_str_eq(first_task, RSC_DEMOTE, TRUE) == FALSE) {
+                    crm_err("Internal error: No action found for %s in %s (first)",
+                            first_task, first_child->id);
 
-            if (then_action == NULL && is_not_set(then_child->flags, pe_rsc_orphan)
-                && crm_str_eq(then->task, RSC_STOP, TRUE) == FALSE
-                && crm_str_eq(then->task, RSC_DEMOTED, TRUE) == FALSE) {
-                crm_err("Internal error: No action found for %s in %s (then)", then->task,
-                        then_child->id);
-            }
-
-            if (first_action == NULL || then_action == NULL) {
+                } else {
+                    crm_trace("No action found for %s in %s%s (first)",
+                              first_task, first_child->id,
+                              is_set(first_child->flags, pe_rsc_orphan) ? " (ORPHAN)" : "");
+                }
                 continue;
             }
+
+            /* We're only interested if 'then' is neither stopping nor being demoted */ 
+            if (then_action == NULL) {
+                if (is_not_set(then_child->flags, pe_rsc_orphan)
+                    && crm_str_eq(then->task, RSC_STOP, TRUE) == FALSE
+                    && crm_str_eq(then->task, RSC_DEMOTE, TRUE) == FALSE) {
+                    crm_err("Internal error: No action found for %s in %s (then)",
+                            then->task, then_child->id);
+
+                } else {
+                    crm_trace("No action found for %s in %s%s (then)",
+                              then->task, then_child->id,
+                              is_set(then_child->flags, pe_rsc_orphan) ? " (ORPHAN)" : "");
+                }
+                continue;
+            }
+
             if (order_actions(first_action, then_action, type)) {
                 crm_debug("Created constraint for %s -> %s", first_action->uuid, then_action->uuid);
                 changed |= (pe_graph_updated_first | pe_graph_updated_then);
@@ -1317,7 +1346,8 @@ clone_update_actions(action_t * first, action_t * then, node_t * node, enum pe_a
         && then->rsc && then->rsc->variant >= pe_clone) {
         clone_variant_data_t *clone_data = NULL;
 
-        if (strstr(then->uuid, "_stop_0") || strstr(then->uuid, "_demote_0")) {
+        if (crm_ends_with(then->uuid, "_stop_0")
+            || crm_ends_with(then->uuid, "_demote_0")) {
             get_clone_variant_data(clone_data, first->rsc);
             rsc = first->rsc->id;
         } else {
