@@ -47,13 +47,13 @@ get_action_flags(action_t * action, node_t * node)
 
             /* Go to great lengths to ensure the correct value for pe_action_runnable...
              *
-             * If we are a clone, then for _ordering_ constraints, its only relevant
+             * If we are a clone, then for _ordering_ constraints, it's only relevant
              * if we are runnable _anywhere_.
              *
              * This only applies to _runnable_ though, and only for ordering constraints.
              * If this function is ever used during colocation, then we'll need additional logic
              *
-             * Not very satisfying, but its logical and appears to work well.
+             * Not very satisfying, but it's logical and appears to work well.
              */
             if (is_not_set(clone_flags, pe_action_runnable)
                 && is_set(flags, pe_action_runnable)) {
@@ -154,7 +154,7 @@ rsc_expand_action(action_t * action)
         gboolean notify = FALSE;
 
         if (action->rsc->parent == NULL) {
-            /* Only outter-most resources have notification actions */
+            /* Only outermost resources have notification actions */
             notify = is_set(action->rsc->flags, pe_rsc_notify);
         }
 
@@ -190,7 +190,7 @@ graph_update_action(action_t * first, action_t * then, node_t * node, enum pe_ac
                                                 pe_action_optional, pe_order_implies_then);
 
         } else if (is_set(flags, pe_action_optional) == FALSE) {
-            if (update_action_flags(then, pe_action_optional | pe_action_clear)) {
+            if (update_action_flags(then, pe_action_optional | pe_action_clear, __FUNCTION__)) {
                 changed |= pe_graph_updated_then;
             }
         }
@@ -222,7 +222,8 @@ graph_update_action(action_t * first, action_t * then, node_t * node, enum pe_ac
                                                  pe_action_optional, pe_order_implies_first);
 
         } else if (is_set(flags, pe_action_optional) == FALSE) {
-            if (update_action_flags(first, pe_action_runnable | pe_action_clear)) {
+            pe_rsc_trace(first->rsc, "first unrunnable: %s then %s", first->uuid, then->uuid);
+            if (update_action_flags(first, pe_action_runnable | pe_action_clear, __FUNCTION__)) {
                 changed |= pe_graph_updated_first;
             }
         }
@@ -267,7 +268,7 @@ graph_update_action(action_t * first, action_t * then, node_t * node, enum pe_ac
             /* if the runnable before count for then exceeds the required number
              * of "before" runnable actions... mark then as runnable */
             if (then->runnable_before >= then->required_runnable_before) {
-                if (update_action_flags(then, pe_action_runnable)) {
+                if (update_action_flags(then, pe_action_runnable, __FUNCTION__)) {
                     changed |= pe_graph_updated_then;
                 }
             }
@@ -288,7 +289,8 @@ graph_update_action(action_t * first, action_t * then, node_t * node, enum pe_ac
                                                 pe_action_runnable, pe_order_runnable_left);
 
         } else if (is_set(flags, pe_action_runnable) == FALSE) {
-            if (update_action_flags(then, pe_action_runnable | pe_action_clear)) {
+            pe_rsc_trace(then->rsc, "then unrunnable: %s then %s", first->uuid, then->uuid);
+            if (update_action_flags(then, pe_action_runnable | pe_action_clear, __FUNCTION__)) {
                 changed |= pe_graph_updated_then;
             }
         }
@@ -361,13 +363,13 @@ graph_update_action(action_t * first, action_t * then, node_t * node, enum pe_ac
         && (flags & pe_action_optional) == 0) {
         processed = TRUE;
         crm_trace("%s implies %s printed", first->uuid, then->uuid);
-        update_action_flags(then, pe_action_print_always);      /* dont care about changed */
+        update_action_flags(then, pe_action_print_always, __FUNCTION__);      /* dont care about changed */
     }
 
     if ((type & pe_order_implies_first_printed) && (flags & pe_action_optional) == 0) {
         processed = TRUE;
         crm_trace("%s implies %s printed", then->uuid, first->uuid);
-        update_action_flags(first, pe_action_print_always);     /* dont care about changed */
+        update_action_flags(first, pe_action_print_always, __FUNCTION__);     /* dont care about changed */
     }
 
     if ((type & pe_order_implies_then
@@ -379,7 +381,7 @@ graph_update_action(action_t * first, action_t * then, node_t * node, enum pe_ac
         && is_set(first->rsc->flags, pe_rsc_block)
         && is_not_set(first->flags, pe_action_runnable)) {
 
-        if (update_action_flags(then, pe_action_runnable | pe_action_clear)) {
+        if (update_action_flags(then, pe_action_runnable | pe_action_clear, __FUNCTION__)) {
             changed |= pe_graph_updated_then;
         }
 
@@ -509,6 +511,17 @@ update_action(action_t * then)
             }
         }
 
+        /* Disable constraint if it only applies when on same node, but isn't */
+        if (is_set(other->type, pe_order_same_node) && first_node && then_node
+            && (first_node->details != then_node->details)) {
+
+            crm_trace("Disabled constraint %s on %s -> %s on %s",
+                       other->action->uuid, first_node->details->uname,
+                       then->uuid, then_node->details->uname);
+            other->type = pe_order_none;
+            continue;
+        }
+
         clear_bit(changed, pe_graph_updated_first);
 
         if (first->rsc != then->rsc
@@ -516,7 +529,7 @@ update_action(action_t * then)
             first = rsc_expand_action(first);
         }
         if (first != other->action) {
-            crm_trace("Ordering %s afer %s instead of %s", then->uuid, first->uuid,
+            crm_trace("Ordering %s after %s instead of %s", then->uuid, first->uuid,
                       other->action->uuid);
         }
 
@@ -683,33 +696,33 @@ shutdown_constraints(node_t * node, action_t * shutdown_op, pe_working_set_t * d
     return TRUE;
 }
 
+/*!
+ * \internal
+ * \brief Order all actions appropriately relative to a fencing operation
+ *
+ * Ensure start operations of affected resources are ordered after fencing,
+ * imply stop and demote operations of affected resources by marking them as
+ * pseudo-actions, etc.
+ *
+ * \param[in]     node        Node to be fenced
+ * \param[in]     stonith_op  Fencing operation
+ * \param[in/out] data_set    Working set of cluster
+ */
 gboolean
 stonith_constraints(node_t * node, action_t * stonith_op, pe_working_set_t * data_set)
 {
+    GListPtr r = NULL;
+
     CRM_CHECK(stonith_op != NULL, return FALSE);
+    for (r = data_set->resources; r != NULL; r = r->next) {
+        resource_t *rsc = (resource_t *) r->data;
 
-    /*
-     * Make sure the stonith OP occurs before we start any shared resources
-     */
-    if (stonith_op != NULL) {
-        GListPtr lpc = NULL;
+        if ((stonith_op->rsc == NULL)
+            || ((stonith_op->rsc != rsc) && (stonith_op->rsc != rsc->container))) {
 
-        for (lpc = data_set->resources; lpc != NULL; lpc = lpc->next) {
-            resource_t *rsc = (resource_t *) lpc->data;
-
-            if(stonith_op->rsc == NULL) {
-                rsc_stonith_ordering(rsc, stonith_op, data_set);
-
-            } else if(stonith_op->rsc != rsc && stonith_op->rsc != rsc->container) {
-                rsc_stonith_ordering(rsc, stonith_op, data_set);
-            }
+            rsc_stonith_ordering(rsc, stonith_op, data_set);
         }
     }
-
-    /* add the stonith OP as a stop pre-req and the mark the stop
-     * as a pseudo op - since its now redundant
-     */
-
     return TRUE;
 }
 
@@ -774,13 +787,101 @@ get_router_node(action_t *action)
     return router_node;
 }
 
+/*!
+ * \internal
+ * \brief Add an XML node tag for a specified ID
+ *
+ * \param[in]     id      Node UUID to add
+ * \param[in,out] xml     Parent XML tag to add to
+ */
+static void
+add_node_to_xml_by_id(const char *id, xmlNode *xml)
+{
+    xmlNode *node_xml;
+
+    node_xml = create_xml_node(xml, XML_CIB_TAG_NODE);
+    crm_xml_add(node_xml, XML_ATTR_UUID, id);
+}
+
+/*!
+ * \internal
+ * \brief Add an XML node tag for a specified node
+ *
+ * \param[in]     node  Node to add
+ * \param[in/out] xml   XML to add node to
+ */
+static void
+add_node_to_xml(const node_t *node, void *xml)
+{
+    add_node_to_xml_by_id(node->details->id, (xmlNode *) xml);
+}
+
+/*!
+ * \internal
+ * \brief Add XML with nodes that an action is expected to bring down
+ *
+ * If a specified action is expected to bring any nodes down, add an XML block
+ * with their UUIDs. When a node is lost, this allows the crmd to determine
+ * whether it was expected.
+ *
+ * \param[in,out] xml       Parent XML tag to add to
+ * \param[in]     action    Action to check for downed nodes
+ * \param[in]     data_set  Working set for cluster
+ */
+static void
+add_downed_nodes(xmlNode *xml, const action_t *action,
+                 const pe_working_set_t *data_set)
+{
+    CRM_CHECK(xml && action && action->node && data_set, return);
+
+    if (safe_str_eq(action->task, CRM_OP_SHUTDOWN)) {
+
+        /* Shutdown makes the action's node down */
+        xmlNode *downed = create_xml_node(xml, XML_GRAPH_TAG_DOWNED);
+        add_node_to_xml_by_id(action->node->details->id, downed);
+
+    } else if (safe_str_eq(action->task, CRM_OP_FENCE)) {
+
+        /* Fencing makes the action's node and any hosted guest nodes down */
+        const char *fence = g_hash_table_lookup(action->meta, "stonith_action");
+
+        if (safe_str_eq(fence, "off") || safe_str_eq(fence, "reboot")) {
+            xmlNode *downed = create_xml_node(xml, XML_GRAPH_TAG_DOWNED);
+            add_node_to_xml_by_id(action->node->details->id, downed);
+            pe_foreach_guest_node(data_set, action->node, add_node_to_xml, downed);
+        }
+
+    } else if (action->rsc && action->rsc->is_remote_node
+               && safe_str_eq(action->task, CRMD_ACTION_STOP)) {
+
+        /* Stopping a remote connection resource makes connected node down,
+         * unless it's part of a migration
+         */
+        GListPtr iter;
+        action_t *input;
+        gboolean migrating = FALSE;
+
+        for (iter = action->actions_before; iter != NULL; iter = iter->next) {
+            input = ((action_wrapper_t *) iter->data)->action;
+            if (input->rsc && safe_str_eq(action->rsc->id, input->rsc->id)
+               && safe_str_eq(input->task, CRMD_ACTION_MIGRATED)) {
+                migrating = TRUE;
+                break;
+            }
+        }
+        if (!migrating) {
+            xmlNode *downed = create_xml_node(xml, XML_GRAPH_TAG_DOWNED);
+            add_node_to_xml_by_id(action->rsc->id, downed);
+        }
+    }
+}
+
 static xmlNode *
 action2xml(action_t * action, gboolean as_input, pe_working_set_t *data_set)
 {
     gboolean needs_node_info = TRUE;
     xmlNode *action_xml = NULL;
     xmlNode *args_xml = NULL;
-    char *action_id_s = NULL;
 
     if (action == NULL) {
         return NULL;
@@ -788,7 +889,6 @@ action2xml(action_t * action, gboolean as_input, pe_working_set_t *data_set)
 
     if (safe_str_eq(action->task, CRM_OP_FENCE)) {
         action_xml = create_xml_node(NULL, XML_GRAPH_TAG_CRM_EVENT);
-/* 		needs_node_info = FALSE; */
 
     } else if (safe_str_eq(action->task, CRM_OP_SHUTDOWN)) {
         action_xml = create_xml_node(NULL, XML_GRAPH_TAG_CRM_EVENT);
@@ -810,9 +910,7 @@ action2xml(action_t * action, gboolean as_input, pe_working_set_t *data_set)
         action_xml = create_xml_node(NULL, XML_GRAPH_TAG_RSC_OP);
     }
 
-    action_id_s = crm_itoa(action->id);
-    crm_xml_add(action_xml, XML_ATTR_ID, action_id_s);
-    free(action_id_s);
+    crm_xml_add_int(action_xml, XML_ATTR_ID, action->id);
 
     crm_xml_add(action_xml, XML_LRM_ATTR_TASK, action->task);
     if (action->rsc != NULL && action->rsc->clone_name != NULL) {
@@ -854,10 +952,12 @@ action2xml(action_t * action, gboolean as_input, pe_working_set_t *data_set)
         }
     }
 
+    /* No details if this action is only being listed in the inputs section */
     if (as_input) {
         return action_xml;
     }
 
+    /* List affected resource */
     if (action->rsc) {
         if (is_set(action->flags, pe_action_pseudo) == FALSE) {
             int lpc = 0;
@@ -925,6 +1025,7 @@ action2xml(action_t * action, gboolean as_input, pe_working_set_t *data_set)
         }
     }
 
+    /* List any attributes in effect */
     args_xml = create_xml_node(NULL, XML_TAG_ATTRS);
     crm_xml_add(args_xml, XML_ATTR_CRM_VERSION, CRM_FEATURE_SET);
 
@@ -962,9 +1063,14 @@ action2xml(action_t * action, gboolean as_input, pe_working_set_t *data_set)
     }
 
     sorted_xml(args_xml, action_xml, FALSE);
-    crm_log_xml_trace(action_xml, "dumped action");
     free_xml(args_xml);
 
+    /* List any nodes this action is expected to make down */
+    if (needs_node_info && (action->node != NULL)) {
+        add_downed_nodes(action_xml, action, data_set);
+    }
+
+    crm_log_xml_trace(action_xml, "dumped action");
     return action_xml;
 }
 
@@ -1164,7 +1270,7 @@ check_dump_input(int last_action, action_t * action, action_wrapper_t * wrapper)
         return FALSE;
 
     } else if ((wrapper->type == pe_order_optional)
-               && strstr(wrapper->action->uuid, "_stop_0")
+               && crm_ends_with(wrapper->action->uuid, "_stop_0")
                && is_set(wrapper->action->flags, pe_action_migrate_runnable)) {
 
         /* for optional only ordering, ordering is not preserved for
@@ -1234,7 +1340,7 @@ check_dump_input(int last_action, action_t * action, action_wrapper_t * wrapper)
                && wrapper->action->rsc != action->rsc
                && is_set(wrapper->action->rsc->flags, pe_rsc_failed)
                && is_not_set(wrapper->action->rsc->flags, pe_rsc_managed)
-               && strstr(wrapper->action->uuid, "_stop_0")
+               && crm_ends_with(wrapper->action->uuid, "_stop_0")
                && action->rsc && action->rsc->variant >= pe_clone) {
         crm_warn("Ignoring requirement that %s complete before %s:"
                  " unmanaged failed resources cannot prevent clone shutdown",
