@@ -20,6 +20,7 @@
 
 #include <crm/msg_xml.h>
 #include <allocate.h>
+#include <notif.h>
 #include <utils.h>
 
 #define VARIANT_CLONE 1
@@ -428,7 +429,7 @@ filter_anonymous_instance(resource_t * rsc, node_t * node)
         resource_t *child = rIter->data;
 
         /*
-         * We know its not running, but any score will still count if
+         * We know it's not running, but any score will still count if
          * the instance has been probed on $node
          *
          * Again use ->find_rsc() because we might be a cloned group
@@ -475,7 +476,7 @@ master_score(resource_t * rsc, node_t * node, int not_set_value)
 
     if (node == NULL) {
         if (rsc->fns->state(rsc, TRUE) < RSC_ROLE_STARTED) {
-            pe_rsc_trace(rsc, "Ingoring master score for %s: unknown state", rsc->id);
+            pe_rsc_trace(rsc, "Ignoring master score for %s: unknown state", rsc->id);
             return score;
         }
 
@@ -844,14 +845,14 @@ master_create_actions(resource_t * rsc, pe_working_set_t * data_set)
                                     RSC_PROMOTED, NULL, !any_promoting, TRUE, data_set);
 
     action_complete->priority = INFINITY;
-    update_action_flags(action, pe_action_pseudo);
-    update_action_flags(action, pe_action_runnable);
-    update_action_flags(action_complete, pe_action_pseudo);
-    update_action_flags(action_complete, pe_action_runnable);
+    update_action_flags(action, pe_action_pseudo, __FUNCTION__);
+    update_action_flags(action, pe_action_runnable, __FUNCTION__);
+    update_action_flags(action_complete, pe_action_pseudo, __FUNCTION__);
+    update_action_flags(action_complete, pe_action_runnable, __FUNCTION__);
 
     if (clone_data->masters_allocated > 0) {
-        update_action_flags(action, pe_action_runnable);
-        update_action_flags(action_complete, pe_action_runnable);
+        update_action_flags(action, pe_action_runnable, __FUNCTION__);
+        update_action_flags(action_complete, pe_action_runnable, __FUNCTION__);
     }
 
     child_promoting_constraints(clone_data, pe_order_optional,
@@ -868,10 +869,10 @@ master_create_actions(resource_t * rsc, pe_working_set_t * data_set)
                                     RSC_DEMOTED, NULL, !any_demoting, TRUE, data_set);
     action_complete->priority = INFINITY;
 
-    update_action_flags(action, pe_action_pseudo);
-    update_action_flags(action, pe_action_runnable);
-    update_action_flags(action_complete, pe_action_pseudo);
-    update_action_flags(action_complete, pe_action_runnable);
+    update_action_flags(action, pe_action_pseudo, __FUNCTION__);
+    update_action_flags(action, pe_action_runnable, __FUNCTION__);
+    update_action_flags(action_complete, pe_action_pseudo, __FUNCTION__);
+    update_action_flags(action_complete, pe_action_runnable, __FUNCTION__);
 
     child_demoting_constraints(clone_data, pe_order_optional, rsc, NULL, last_demote_rsc, data_set);
 
@@ -1029,7 +1030,7 @@ master_rsc_colocation_rh(resource_t * rsc_lh, resource_t * rsc_rh, rsc_colocatio
             }
         }
 
-        /* Only do this if its not a master-master colocation
+        /* Only do this if it's not a master-master colocation
          * Doing this unconditionally would prevent the slaves from being started
          */
         if (constraint->role_lh != RSC_ROLE_MASTER || constraint->role_rh != RSC_ROLE_MASTER) {
