@@ -40,7 +40,7 @@ cib_t *global_cib = NULL;
 GListPtr op_fail = NULL;
 bool action_numbers = FALSE;
 gboolean quiet = FALSE;
-gboolean print_pending = FALSE;
+gboolean print_pending = TRUE;
 char *temp_shadow = NULL;
 extern gboolean bringing_nodes_online;
 
@@ -468,7 +468,7 @@ static struct crm_option long_options[] = {
     {"show-scores",   0, 0, 's', "Show allocation scores"},
     {"show-utilization",   0, 0, 'U', "Show utilization information"},
     {"profile",       1, 0, 'P', "Run all tests in the named directory to create profiling data"},
-    {"pending",       0, 0, 'j', "\tDisplay pending state if 'record-pending' is enabled"},
+    {"pending",       0, 0, 'j', "\tDisplay pending state if 'record-pending' is enabled", pcmk_option_hidden},
 
     {"-spacer-",     0, 0, '-', "\nSynthetic Cluster Events:"},
     {"node-up",      1, 0, 'u', "\tBring a node online"},
@@ -565,7 +565,7 @@ profile_all(const char *dir)
                 free(namelist[file_num]);
                 continue;
 
-            } else if (strstr(namelist[file_num]->d_name, ".xml") == NULL) {
+            } else if (!crm_ends_with(namelist[file_num]->d_name, ".xml")) {
                 free(namelist[file_num]);
                 continue;
             }
@@ -583,6 +583,26 @@ profile_all(const char *dir)
     return lpc;
 }
 
+static int
+count_resources(pe_working_set_t * data_set, resource_t * rsc)
+{
+    int count = 0;
+    GListPtr gIter = NULL;
+
+    if (rsc == NULL) {
+        gIter = data_set->resources;
+    } else if (rsc->children) {
+        gIter = rsc->children;
+    } else {
+        return is_not_set(rsc->flags, pe_rsc_orphan);
+    }
+
+    for (; gIter != NULL; gIter = gIter->next) {
+        count += count_resources(data_set, gIter->data);
+    }
+    return count;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -798,6 +818,17 @@ main(int argc, char **argv)
     if (quiet == FALSE) {
         int options = print_pending ? pe_print_pending : 0;
 
+        if(is_set(data_set.flags, pe_flag_maintenance_mode)) {
+            quiet_log("\n              *** Resource management is DISABLED ***");
+            quiet_log("\n  The cluster will not attempt to start, stop or recover services");
+            quiet_log("\n");
+        }
+
+        if(data_set.disabled_resources || data_set.blocked_resources) {
+            quiet_log("%d of %d resources DISABLED and %d BLOCKED from being started due to failures\n",
+                      data_set.disabled_resources, count_resources(&data_set, NULL), data_set.blocked_resources);
+        }
+
         quiet_log("\nCurrent cluster status:\n");
         print_cluster_status(&data_set, options);
     }
